@@ -27,10 +27,16 @@
 
 from .qtuic import uic
 import pyqtgraph as _pg
-from pyqtgraph import QtCore, QtGui
+from pyqtgraph import QtCore
 import os
 import json
 import logging
+from .sardanaUtils import numpyEncoder
+
+try:
+    from pyqtgraph import QtWidgets
+except Exception:
+    from pyqtgraph import QtGui as QtWidgets
 
 from . import edDictDialog
 
@@ -41,7 +47,7 @@ _formclass, _baseclass = uic.loadUiType(
 logger = logging.getLogger("lavue")
 
 
-class TableWidgetDragCheckBoxes(QtGui.QTableWidget):
+class TableWidgetDragCheckBoxes(QtWidgets.QTableWidget):
     """ TableWidget with drag and drop checkboxes """
 
     def __init__(self, parent):
@@ -50,7 +56,7 @@ class TableWidgetDragCheckBoxes(QtGui.QTableWidget):
         :param parent: parent object
         :type parent: :class:`pyqtgraph.QtCore.QObject`
         """
-        QtGui.QTableWidget.__init__(self, parent)
+        QtWidgets.QTableWidget.__init__(self, parent)
 
         #: (:obj:`list` < [ :obj:`str`, :obj:`bool` ] > ) checkbox item list
         self.__checkboxdata = []
@@ -64,11 +70,11 @@ class TableWidgetDragCheckBoxes(QtGui.QTableWidget):
         self.setDragDropOverwriteMode(False)
 
         self.setDragDropMode(
-            QtGui.QAbstractItemView.InternalMove)
+            QtWidgets.QAbstractItemView.InternalMove)
         self.setSelectionMode(
-            QtGui.QAbstractItemView.ExtendedSelection)
+            QtWidgets.QAbstractItemView.ExtendedSelection)
         self.setSelectionBehavior(
-            QtGui.QAbstractItemView.SelectRows)
+            QtWidgets.QAbstractItemView.SelectRows)
         self.setColumnCount(1)
         self.horizontalHeader().hide()
         self.verticalHeader().hide()
@@ -102,7 +108,7 @@ class TableWidgetDragCheckBoxes(QtGui.QTableWidget):
         for i, (name, checked) in enumerate(self.__checkboxdata):
             nm = str(name)
             iname = itemnames[nm] if nm in itemnames else nm
-            item = QtGui.QTableWidgetItem(iname)
+            item = QtWidgets.QTableWidgetItem(iname)
             item.setFlags(item.flags() ^ QtCore.Qt.ItemIsEditable)
             item.setCheckState(QtCore.Qt.Checked
                                if checked else QtCore.Qt.Unchecked)
@@ -147,7 +153,7 @@ class TableWidgetDragCheckBoxes(QtGui.QTableWidget):
                 selrows = sorted(
                     set(item.row() for item in self.selectedItems()))
                 tomove = [
-                    QtGui.QTableWidgetItem(self.item(idx, 0))
+                    QtWidgets.QTableWidgetItem(self.item(idx, 0))
                     for idx in selrows
                 ]
                 for idx in reversed(selrows):
@@ -160,7 +166,7 @@ class TableWidgetDragCheckBoxes(QtGui.QTableWidget):
                     self.insertRow(idx)
                     self.setItem(idx, 0, data)
                 event.accept()
-        QtGui.QTableWidget.dropEvent(self, event)
+        QtWidgets.QTableWidget.dropEvent(self, event)
 
     def _target_row(self, event):
         """ provides target row index to drop
@@ -176,7 +182,7 @@ class TableWidgetDragCheckBoxes(QtGui.QTableWidget):
         return idx.row()
 
 
-class ConfigDialog(QtGui.QDialog):
+class ConfigDialog(QtWidgets.QDialog):
 
     def __init__(self, parent=None):
         """ constructor
@@ -184,16 +190,16 @@ class ConfigDialog(QtGui.QDialog):
         :param parent: parent object
         :type parent: :class:`pyqtgraph.QtCore.QObject`
         """
-        QtGui.QDialog.__init__(self, parent)
+        QtWidgets.QDialog.__init__(self, parent)
 
         #: (:class:`Ui_ConfigDialog') ui_dialog object from qtdesigner
         self.__ui = _formclass()
         self.__ui.setupUi(self)
-        self.__ui.isLayout = QtGui.QGridLayout(self.__ui.isWidget)
+        self.__ui.isLayout = QtWidgets.QGridLayout(self.__ui.isWidget)
         self.__ui.isLayout.setContentsMargins(0, 0, 0, 0)
         self.__ui.isTable = TableWidgetDragCheckBoxes(self)
         self.__ui.isLayout.addWidget(self.__ui.isTable)
-        self.__ui.twLayout = QtGui.QGridLayout(self.__ui.twWidget)
+        self.__ui.twLayout = QtWidgets.QGridLayout(self.__ui.twWidget)
         self.__ui.twLayout.setContentsMargins(0, 0, 0, 0)
         self.__ui.twTable = TableWidgetDragCheckBoxes(self)
         self.__ui.twLayout.addWidget(self.__ui.twTable)
@@ -206,8 +212,12 @@ class ConfigDialog(QtGui.QDialog):
         self.addrois = False
         #: (:obj:`bool`) fetch rois order enabled
         self.orderrois = False
+        #: (:obj:`bool`) fetch typed rois enabled
+        self.typedrois = False
         #: (:obj:`bool`) security stream enabled
         self.secstream = False
+        #: (:obj:`bool`) zmq colon configuration separator
+        self.zmqcolon = False
         #: (:obj:`bool`) map image sources to color channels
         self.imagechannels = False
         #: (:obj:`str`) security stream port
@@ -216,9 +226,11 @@ class ConfigDialog(QtGui.QDialog):
         self.hidraport = '["50001"]'
         #: (:obj:`bool`) find security stream port automatically
         self.secautoport = True
-        #: (:obj:`float`) refresh rate
-        self.refreshrate = 0.2
-        #: (:obj:`float`) tool refresh rate time is s
+        #: (:obj:`float`) refresh time in s
+        self.refreshtime = 0.2
+        #: (:obj:`bool`) auto enlarge refresh time
+        self.autorefreshtime = True
+        #: (:obj:`float`) tool refresh time is s
         self.toolrefreshtime = 0.02
         #: (:obj:`float`) tool polling interval is s
         self.toolpollinginterval = 1.0
@@ -234,10 +246,14 @@ class ConfigDialog(QtGui.QDialog):
         self.showmask = False
         #: (:obj:`bool`) show high value mask widget
         self.showhighvaluemask = False
+        #: (:obj:`bool`) show intensity overflow widget
+        self.showoverflow = False
         #: (:obj:`bool`) show statistics widget
         self.showstats = True
         #: (:obj:`bool`) show image step widget
         self.showsteps = True
+        #: (:obj:`bool`) show user plot widget
+        self.showuserplot = False
         #: (:obj:`bool`) calculate variance
         self.calcvariance = False
         #: (:obj:`bool`) zero mask enabled
@@ -299,6 +315,8 @@ class ConfigDialog(QtGui.QDialog):
         self.asapobeamtime = ""
         #: (:obj:`str`) asapo sourcepath
         self.asaposourcepath = ""
+        #: (:obj:`str`) asapo beamline metadata file
+        self.asapobtmetafile = "/gpfs/current/beamtime-metadata-*.json"
         #: (:obj:`list` < :obj:`str` > ) asapo datasources
         self.asapodatasources = []
 
@@ -360,11 +378,21 @@ class ConfigDialog(QtGui.QDialog):
         self.geometryfromsource = False
         #: (:obj:`str`) json list with filters
         self.filters = "[]"
+        #: (:obj:`str`) json list with user functions
+        self.userfunctions = "[]"
+
         #: (:obj:`str`) json list with rois colors
         self.roiscolors = "[]"
         #: (:obj:`list`<:class:`pyqtgraph.ColorButton`>)
         #    list with rois color widgets
         self.__roiswidgets = []
+
+        #: (:obj:`str`) json list with overflow color
+        self.overflowcolor = "[255, 255, 255]"
+        #: (:class:`pyqtgraph.ColorButton`)
+        #    list with rois color widgets
+        self.__overflowcolorwidget = None
+
         #: (:obj:`bool`) show all rois flag
         self.showallrois = False
         #: (:obj:`bool`) send rois to LavueController flag
@@ -373,6 +401,8 @@ class ConfigDialog(QtGui.QDialog):
         self.sendresults = False
         #: (:obj:`bool`) set roi1, roi2, roi3, ... when alias names are missing
         self.singlerois = False
+        #: (:obj:`bool`) display roi aliases on the 2d image
+        self.labelrois = False
         #: (:obj:`int`) number of points for diffractogram
         self.diffnpt = 1000
         #: (:obj:`bool`) correct solid angle flag
@@ -426,14 +456,15 @@ class ConfigDialog(QtGui.QDialog):
                 for key in list(record.keys()):
                     if not str(key).strip():
                         record.pop(key)
-                obj.setText(json.dumps(record))
+                obj.setText(json.dumps(record, cls=numpyEncoder))
             return True
         return False
 
     def createGUI(self):
         """ create GUI
         """
-        self.__ui.rateDoubleSpinBox.setValue(self.refreshrate)
+        self.__ui.rateDoubleSpinBox.setValue(self.refreshtime)
+        self.__ui.enlargeCheckBox.setChecked(self.autorefreshtime)
         self.__ui.nrsourcesSpinBox.setValue(self.nrsources)
         self.__ui.diffsizeSpinBox.setValue(self.diffnpt)
         self.__ui.toolrefreshtimeDoubleSpinBox.setValue(self.toolrefreshtime)
@@ -450,8 +481,10 @@ class ConfigDialog(QtGui.QDialog):
         self.__ui.doorLineEdit.setText(self.door)
         self.__ui.addroisCheckBox.setChecked(self.addrois)
         self.__ui.orderroisCheckBox.setChecked(self.orderrois)
+        self.__ui.typedroisCheckBox.setChecked(self.typedrois)
         self.__ui.imagechannelsCheckBox.setChecked(self.imagechannels)
         self.__ui.secstreamCheckBox.setChecked(self.secstream)
+        self.__ui.zmqcolonCheckBox.setChecked(self.zmqcolon)
         self.__ui.zeromaskCheckBox.setChecked(self.zeromask)
         self.__ui.nanmaskCheckBox.setChecked(self.nanmask)
         self.__ui.negmaskCheckBox.setChecked(self.negmask)
@@ -465,8 +498,10 @@ class ConfigDialog(QtGui.QDialog):
         self.__ui.showaddhistoCheckBox.setChecked(self.showaddhisto)
         self.__ui.showmaskCheckBox.setChecked(self.showmask)
         self.__ui.showmaskhighCheckBox.setChecked(self.showhighvaluemask)
+        self.__ui.showoverflowCheckBox.setChecked(self.showoverflow)
         self.__ui.showstatsCheckBox.setChecked(self.showstats)
         self.__ui.showstepsCheckBox.setChecked(self.showsteps)
+        self.__ui.showuserplotCheckBox.setChecked(self.showuserplot)
         self.__ui.calcvarianceCheckBox.setChecked(self.calcvariance)
         self.__ui.showsubCheckBox.setChecked(self.showsub)
         self.__ui.shownormCheckBox.setChecked(self.shownorm)
@@ -489,6 +524,7 @@ class ConfigDialog(QtGui.QDialog):
         self.__ui.asapotokenLineEdit.setText(self.asapotoken)
         self.__ui.asapobeamtimeLineEdit.setText(self.asapobeamtime)
         self.__ui.asaposourcepathLineEdit.setText(self.asaposourcepath)
+        self.__ui.asapobtmetafileLineEdit.setText(self.asapobtmetafile)
         self.__ui.defdetserversCheckBox.setChecked(self.defdetservers)
         self.__ui.autozmqtopicsCheckBox.setChecked(self.autozmqtopics)
         self.__ui.interruptCheckBox.setChecked(self.interruptonerror)
@@ -510,6 +546,7 @@ class ConfigDialog(QtGui.QDialog):
         self.__ui.sendroisCheckBox.setChecked(self.sendrois)
         self.__ui.sendresultsCheckBox.setChecked(self.sendresults)
         self.__ui.singleroisCheckBox.setChecked(self.singlerois)
+        self.__ui.labelroisCheckBox.setChecked(self.labelrois)
         self.__ui.showallroisCheckBox.setChecked(self.showallrois)
         self.__ui.sourcedisplayCheckBox.setChecked(self.sourcedisplay)
         self.__ui.crosshairCheckBox.setChecked(self.crosshairlocker)
@@ -556,7 +593,9 @@ class ConfigDialog(QtGui.QDialog):
             self._removeROIColorWidget)
 
         self.__setROIsColorsWidgets()
+        self.__setOverflowColorWidget()
         self.__setFiltersGroupBox()
+        self.__setUserFunctionsGroupBox()
         self.__ui.isTable.create(
             json.loads(self.imagesources), self.availimagesources,
             self.imagesourcenames
@@ -570,11 +609,11 @@ class ConfigDialog(QtGui.QDialog):
         """ updates filter tab  widget
         """
         self.__ui.addupPushButton = self.__ui.filterButtonBox.addButton(
-            "Insert Row &Above", QtGui.QDialogButtonBox.ActionRole)
+            "Insert Row &Above", QtWidgets.QDialogButtonBox.ActionRole)
         self.__ui.adddownPushButton = self.__ui.filterButtonBox.addButton(
-            "Insert Row &Below", QtGui.QDialogButtonBox.ActionRole)
+            "Insert Row &Below", QtWidgets.QDialogButtonBox.ActionRole)
         self.__ui.removePushButton = self.__ui.filterButtonBox.addButton(
-            "&Delete Row", QtGui.QDialogButtonBox.ActionRole)
+            "&Delete Row", QtWidgets.QDialogButtonBox.ActionRole)
         self.__populateTable(0)
         self.__ui.addupPushButton.clicked.connect(self.__addup)
         self.__ui.adddownPushButton.clicked.connect(self.__adddown)
@@ -585,13 +624,16 @@ class ConfigDialog(QtGui.QDialog):
     def __updateRecord(self):
         fltlist = []
         for i in range(self.__ui.filterTableWidget.rowCount()):
+            checked = True
             item = self.__ui.filterTableWidget.item(i, 0)
             if item is not None:
                 fltname = item.data(QtCore.Qt.EditRole)
                 if hasattr(fltname, "toString"):
                     fltname = fltname.toString()
+                checked = (item.checkState() != QtCore.Qt.Unchecked)
             else:
                 fltname = ""
+                checked = False
             item2 = self.__ui.filterTableWidget.item(i, 1)
             if item2 is not None:
                 params = item2.data(QtCore.Qt.EditRole)
@@ -599,8 +641,8 @@ class ConfigDialog(QtGui.QDialog):
                     params = params.toString()
             else:
                 params = ""
-            fltlist.append([fltname or "", params or ""])
-        filters = json.dumps(fltlist)
+            fltlist.append([fltname or "", params or "", checked])
+        filters = json.dumps(fltlist, cls=numpyEncoder)
         if self.filters != filters:
             self.filters = filters
             return True
@@ -611,7 +653,7 @@ class ConfigDialog(QtGui.QDialog):
         """ changes the current value of the variable
 
         :param item: current item
-        :type item: :class:`QtGui.QTableWidgetItem`
+        :type item: :class:`QtWidgets.QTableWidgetItem`
         """
         self.__updateRecord()
 
@@ -622,11 +664,11 @@ class ConfigDialog(QtGui.QDialog):
         row = self.__ui.filterTableWidget.currentRow()
         fltlist = json.loads(self.filters)
         if row >= 0 and row <= len(fltlist):
-            fltlist.insert(row, ["", ""])
+            fltlist.insert(row, ["", "", True])
         else:
-            fltlist.insert(0, ["", ""])
+            fltlist.insert(0, ["", "", True])
 
-        self.filters = json.dumps(fltlist)
+        self.filters = json.dumps(fltlist, cls=numpyEncoder)
         self.__populateTable()
         self.__updateRecord()
 
@@ -637,11 +679,11 @@ class ConfigDialog(QtGui.QDialog):
         row = self.__ui.filterTableWidget.currentRow()
         fltlist = json.loads(self.filters)
         if row >= 0 and row <= len(fltlist):
-            fltlist.insert(row + 1, ["", ""])
+            fltlist.insert(row + 1, ["", "", True])
         else:
-            fltlist.append(["", ""])
+            fltlist.append(["", "", True])
 
-        self.filters = json.dumps(fltlist)
+        self.filters = json.dumps(fltlist, cls=numpyEncoder)
         self.__populateTable()
         self.__updateRecord()
 
@@ -652,16 +694,18 @@ class ConfigDialog(QtGui.QDialog):
         row = self.__ui.filterTableWidget.currentRow()
         fltlist = json.loads(self.filters)
         if row >= 0 and row < len(fltlist):
-
-            flt, params = fltlist[row]
-            if QtGui.QMessageBox.question(
+            fltparams = fltlist[row]
+            if len(fltparams) == 2:
+                fltparams.append(True)
+            flt, params, checked = fltparams
+            if QtWidgets.QMessageBox.question(
                     self, "Removing Filter",
                     'Would you like  to remove "%s": "%s" ?' % (flt, params),
-                    QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
-                    QtGui.QMessageBox.Yes) == QtGui.QMessageBox.No:
+                    QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
+                    QtWidgets.QMessageBox.Yes) == QtWidgets.QMessageBox.No:
                 return
             fltlist.pop(row)
-            self.filters = json.dumps(fltlist)
+            self.filters = json.dumps(fltlist, cls=numpyEncoder)
             self.__populateTable()
             self.__updateRecord()
 
@@ -681,30 +725,191 @@ class ConfigDialog(QtGui.QDialog):
         self.__ui.filterTableWidget.setColumnCount(len(headers))
         self.__ui.filterTableWidget.setHorizontalHeaderLabels(headers)
         for row, fltparams in enumerate(fltlist):
-            flt, params = fltparams
-            item = QtGui.QTableWidgetItem(flt or "")
+            if len(fltparams) == 2:
+                fltparams.append(True)
+            flt, params, checked = fltparams
+            item = QtWidgets.QTableWidgetItem(flt or "")
             item.setData(QtCore.Qt.EditRole, (flt or ""))
+            item.setFlags(
+                QtCore.Qt.EditRole |
+                QtCore.Qt.ItemIsUserCheckable |
+                QtCore.Qt.ItemIsEnabled)
+            item.setCheckState(QtCore.Qt.Checked
+                               if checked else QtCore.Qt.Unchecked)
             self.__ui.filterTableWidget.setItem(row, 0, item)
 
-            item2 = QtGui.QTableWidgetItem(params or "")
+            item2 = QtWidgets.QTableWidgetItem(params or "")
             item2.setData(QtCore.Qt.EditRole, (params or ""))
             self.__ui.filterTableWidget.setItem(row, 1, item2)
         self.__ui.filterTableWidget.resizeColumnsToContents()
         self.__ui.filterTableWidget.setSelectionMode(
-            QtGui.QAbstractItemView.SingleSelection)
+            QtWidgets.QAbstractItemView.SingleSelection)
         # self.__ui.filterTableWidget.horizontalHeader(
         # ).setStretchLastSection(True)
         if hasattr(self.__ui.filterTableWidget.horizontalHeader(),
                    "setSectionResizeMode"):
             self.__ui.filterTableWidget.horizontalHeader().\
-                setSectionResizeMode(1, QtGui.QHeaderView.Stretch)
+                setSectionResizeMode(1, QtWidgets.QHeaderView.Stretch)
         else:
             self.__ui.filterTableWidget.horizontalHeader().\
-                setResizeMode(1, QtGui.QHeaderView.Stretch)
+                setResizeMode(1, QtWidgets.QHeaderView.Stretch)
         if sitem is not None:
             sitem.setSelected(True)
             self.__ui.filterTableWidget.setCurrentItem(sitem)
 
+    def __setUserFunctionsGroupBox(self):
+        """ updates filter tab  widget
+        """
+        self.__ui.ufAddupPushButton = self.__ui.userfuncButtonBox.addButton(
+            "Insert Row &Above", QtWidgets.QDialogButtonBox.ActionRole)
+        self.__ui.ufAdddownPushButton = \
+            self.__ui.userfuncButtonBox.addButton(
+                "Insert Row &Below", QtWidgets.QDialogButtonBox.ActionRole)
+        self.__ui.ufRemovePushButton = self.__ui.userfuncButtonBox.addButton(
+            "&Delete Row", QtWidgets.QDialogButtonBox.ActionRole)
+        self.__ufPopulateTable(0)
+        self.__ui.ufAddupPushButton.clicked.connect(self.__ufAddup)
+        self.__ui.ufAdddownPushButton.clicked.connect(self.__ufAdddown)
+        self.__ui.userfuncTableWidget.itemChanged.connect(
+            self.__ufTableItemChanged)
+        self.__ui.ufRemovePushButton.clicked.connect(self.__ufRemove)
+
+    def __ufUpdateRecord(self):
+        ufunclist = []
+        for i in range(self.__ui.userfuncTableWidget.rowCount()):
+            checked = True
+            item = self.__ui.userfuncTableWidget.item(i, 0)
+            if item is not None:
+                ufunname = item.data(QtCore.Qt.EditRole)
+                if hasattr(ufunname, "toString"):
+                    ufunname = ufunname.toString()
+                checked = (item.checkState() != QtCore.Qt.Unchecked)
+            else:
+                ufunname = ""
+                checked = False
+            item2 = self.__ui.userfuncTableWidget.item(i, 1)
+            if item2 is not None:
+                params = item2.data(QtCore.Qt.EditRole)
+                if hasattr(params, "toString"):
+                    params = params.toString()
+            else:
+                params = ""
+            ufunclist.append([ufunname or "", params or "", checked])
+        userfunctions = json.dumps(ufunclist, cls=numpyEncoder)
+        if self.userfunctions != userfunctions:
+            self.userfunctions = userfunctions
+            return True
+        return False
+
+    @QtCore.pyqtSlot("QTableWidgetItem*")
+    def __ufTableItemChanged(self, item):
+        """ changes the current value of the variable
+
+        :param item: current item
+        :type item: :class:`QtWidgets.QTableWidgetItem`
+        """
+        self.__ufUpdateRecord()
+
+    @QtCore.pyqtSlot()
+    def __ufAddup(self):
+        """ adds a new record into the table
+        """
+        row = self.__ui.userfuncTableWidget.currentRow()
+        ufunclist = json.loads(self.userfunctions)
+        if row >= 0 and row <= len(ufunclist):
+            ufunclist.insert(row, ["", "", True])
+        else:
+            ufunclist.insert(0, ["", "", True])
+
+        self.userfunctions = json.dumps(ufunclist, cls=numpyEncoder)
+        self.__ufPopulateTable()
+        self.__ufUpdateRecord()
+
+    @QtCore.pyqtSlot()
+    def __ufAdddown(self):
+        """ adds a new record into the table
+        """
+        row = self.__ui.userfuncTableWidget.currentRow()
+        ufunclist = json.loads(self.userfunctions)
+        if row >= 0 and row <= len(ufunclist):
+            ufunclist.insert(row + 1, ["", "", True])
+        else:
+            ufunclist.append(["", "", True])
+
+        self.userfunctions = json.dumps(ufunclist, cls=numpyEncoder)
+        self.__ufPopulateTable()
+        self.__ufUpdateRecord()
+
+    @QtCore.pyqtSlot()
+    def __ufRemove(self):
+        """ removes the current record from the table
+        """
+        row = self.__ui.userfuncTableWidget.currentRow()
+        ufunclist = json.loads(self.userfunctions)
+        if row >= 0 and row < len(ufunclist):
+            ufparams = ufunclist[row]
+            if len(ufparams) == 2:
+                ufparams.append(True)
+            ufun, params, checked = ufparams
+            if QtWidgets.QMessageBox.question(
+                    self, "Removing Filter",
+                    'Would you like  to remove "%s": "%s" ?' % (ufun, params),
+                    QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
+                    QtWidgets.QMessageBox.Yes) == QtWidgets.QMessageBox.No:
+                return
+            ufunclist.pop(row)
+            self.userfunctions = json.dumps(ufunclist, cls=numpyEncoder)
+            self.__ufPopulateTable()
+            self.__ufUpdateRecord()
+
+    def __ufPopulateTable(self, selected=None):
+        """ populates the group table
+
+        :param selected: selected property
+        :type selected: :obj:`str`
+        """
+        ufunclist = json.loads(self.userfunctions)
+        self.__ui.userfuncTableWidget.clear()
+        sitem = None
+        self.__ui.userfuncTableWidget.setSortingEnabled(False)
+        self.__ui.userfuncTableWidget.setRowCount(len(ufunclist))
+        headers = ["package.module.class or package.module.function",
+                   "initialization parameters"]
+        self.__ui.userfuncTableWidget.setColumnCount(len(headers))
+        self.__ui.userfuncTableWidget.setHorizontalHeaderLabels(headers)
+        for row, ufparams in enumerate(ufunclist):
+            if len(ufparams) == 2:
+                ufparams.append(True)
+            ufun, params, checked = ufparams
+            item = QtWidgets.QTableWidgetItem(ufun or "")
+            item.setData(QtCore.Qt.EditRole, (ufun or ""))
+            item.setFlags(
+                QtCore.Qt.EditRole |
+                QtCore.Qt.ItemIsUserCheckable |
+                QtCore.Qt.ItemIsEnabled)
+            item.setCheckState(QtCore.Qt.Checked
+                               if checked else QtCore.Qt.Unchecked)
+            self.__ui.userfuncTableWidget.setItem(row, 0, item)
+
+            item2 = QtWidgets.QTableWidgetItem(params or "")
+            item2.setData(QtCore.Qt.EditRole, (params or ""))
+            self.__ui.userfuncTableWidget.setItem(row, 1, item2)
+        self.__ui.userfuncTableWidget.resizeColumnsToContents()
+        self.__ui.userfuncTableWidget.setSelectionMode(
+            QtWidgets.QAbstractItemView.SingleSelection)
+        # self.__ui.userfuncTableWidget.horizontalHeader(
+        # ).setStretchLastSection(True)
+        if hasattr(self.__ui.userfuncTableWidget.horizontalHeader(),
+                   "setSectionResizeMode"):
+            self.__ui.userfuncTableWidget.horizontalHeader().\
+                setSectionResizeMode(1, QtWidgets.QHeaderView.Stretch)
+        else:
+            self.__ui.userfuncTableWidget.horizontalHeader().\
+                setResizeMode(1, QtWidgets.QHeaderView.Stretch)
+        if sitem is not None:
+            sitem.setSelected(True)
+            self.__ui.userfuncTableWidget.setCurrentItem(sitem)
+
     def __setROIsColorsWidgets(self):
         """ updates ROIs colors widgets
         """
@@ -733,6 +938,29 @@ class ConfigDialog(QtGui.QDialog):
         self.__roiswidgets.append(cb)
         self.__ui.colorHorizontalLayout.addWidget(cb)
 
+    def __setOverflowColorWidget(self):
+        """ set Mask color widget
+        """
+        try:
+            color = tuple(json.loads(self.overflowcolor))
+        except Exception:
+            color = (255, 255, 255)
+            self.overflowcolor = "[255, 255, 255]"
+        if self.__overflowcolorwidget:
+            cb = self.__overflowcolorwidget
+            self.__overflowcolorwidget = None
+            cb.hide()
+            self.__ui.overflowColorHorizontalLayout.removeWidget(cb)
+        cb = _pg.ColorButton(self, color)
+        self.__overflowcolorwidget = cb
+        self.__ui.overflowColorHorizontalLayout.addWidget(cb)
+
+    def __readOverflowColor(self):
+        """ takes Mask color from mask color widget
+        """
+        color = list(self.__overflowcolorwidget.color(mode='byte')[:3])
+        self.overflowcolor = json.dumps(color, cls=numpyEncoder)
+
     @QtCore.pyqtSlot()
     def _removeROIColorWidget(self):
         """ updates ROIs colors widgets
@@ -747,7 +975,7 @@ class ConfigDialog(QtGui.QDialog):
         colors = []
         for roiswg in self.__roiswidgets:
             colors.append(list(roiswg.color(mode='byte')[:3]))
-        self.roiscolors = json.dumps(colors)
+        self.roiscolors = json.dumps(colors, cls=numpyEncoder)
 
     @QtCore.pyqtSlot(int)
     def _updateSecPortLineEdit(self, value):
@@ -769,15 +997,18 @@ class ConfigDialog(QtGui.QDialog):
         self.door = str(self.__ui.doorLineEdit.text()).strip()
         self.addrois = self.__ui.addroisCheckBox.isChecked()
         self.orderrois = self.__ui.orderroisCheckBox.isChecked()
+        self.typedrois = self.__ui.typedroisCheckBox.isChecked()
         self.imagechannels = self.__ui.imagechannelsCheckBox.isChecked()
         self.hidraport = str(self.__ui.hidraportLineEdit.text()).strip()
         self.maxmbuffersize = str(self.__ui.mbufsizeLineEdit.text()).strip()
         self.secstream = self.__ui.secstreamCheckBox.isChecked()
+        self.zmqcolon = self.__ui.zmqcolonCheckBox.isChecked()
         self.zeromask = self.__ui.zeromaskCheckBox.isChecked()
         self.nanmask = self.__ui.nanmaskCheckBox.isChecked()
         self.negmask = self.__ui.negmaskCheckBox.isChecked()
         self.secautoport = self.__ui.secautoportCheckBox.isChecked()
-        self.refreshrate = float(self.__ui.rateDoubleSpinBox.value())
+        self.refreshtime = float(self.__ui.rateDoubleSpinBox.value())
+        self.autorefreshtime = self.__ui.enlargeCheckBox.isChecked()
         self.floattype = str(self.__ui.floatComboBox.currentText())
         self.toolrefreshtime = float(
             self.__ui.toolrefreshtimeDoubleSpinBox.value())
@@ -802,8 +1033,10 @@ class ConfigDialog(QtGui.QDialog):
         self.showaddhisto = self.__ui.showaddhistoCheckBox.isChecked()
         self.showmask = self.__ui.showmaskCheckBox.isChecked()
         self.showhighvaluemask = self.__ui.showmaskhighCheckBox.isChecked()
+        self.showoverflow = self.__ui.showoverflowCheckBox.isChecked()
         self.showstats = self.__ui.showstatsCheckBox.isChecked()
         self.showsteps = self.__ui.showstepsCheckBox.isChecked()
+        self.showuserplot = self.__ui.showuserplotCheckBox.isChecked()
         self.calcvariance = self.__ui.calcvarianceCheckBox.isChecked()
         self.aspectlocked = self.__ui.aspectlockedCheckBox.isChecked()
         self.autodownsample = self.__ui.downsampleCheckBox.isChecked()
@@ -819,6 +1052,7 @@ class ConfigDialog(QtGui.QDialog):
         self.sendrois = self.__ui.sendroisCheckBox.isChecked()
         self.sendresults = self.__ui.sendresultsCheckBox.isChecked()
         self.singlerois = self.__ui.singleroisCheckBox.isChecked()
+        self.labelrois = self.__ui.labelroisCheckBox.isChecked()
         self.showallrois = self.__ui.showallroisCheckBox.isChecked()
         self.sourcedisplay = self.__ui.sourcedisplayCheckBox.isChecked()
         self.defdetservers = self.__ui.defdetserversCheckBox.isChecked()
@@ -963,9 +1197,12 @@ class ConfigDialog(QtGui.QDialog):
             self.__ui.asapobeamtimeLineEdit.text()).strip()
         self.asaposourcepath = str(
             self.__ui.asaposourcepathLineEdit.text()).strip()
+        self.asapobtmetafile = str(
+            self.__ui.asapobtmetafileLineEdit.text()).strip()
         detservers = str(
             self.__ui.detserversLineEdit.text()).strip().split(" ")
-        self.detservers = json.dumps([ds for ds in detservers if ds])
+        self.detservers = json.dumps(
+            [ds for ds in detservers if ds], cls=numpyEncoder)
         try:
             self.timeout = int(self.__ui.timeoutLineEdit.text())
         except Exception:
@@ -984,7 +1221,8 @@ class ConfigDialog(QtGui.QDialog):
                 self.__ui.hidraportLineEdit.text()).strip().split(" ")
             for pt in hidraport:
                 int(pt)
-            self.hidraport = json.dumps([pt for pt in hidraport if pt])
+            self.hidraport = json.dumps(
+                [pt for pt in hidraport if pt], cls=numpyEncoder)
         except Exception:
             self.__ui.tabWidget.setCurrentIndex(2)
             self.__ui.hidraportLineEdit.setFocus(True)
@@ -998,9 +1236,12 @@ class ConfigDialog(QtGui.QDialog):
             self.__ui.mbufsizeLineEdit.setFocus(True)
             return
         self.__readROIsColors()
+        self.__readOverflowColor()
         self.imagesources = json.dumps(
-            self.__ui.isTable.getChecks(self.availimagesources))
+            self.__ui.isTable.getChecks(self.availimagesources),
+            cls=numpyEncoder)
         self.toolwidgets = json.dumps(
-            self.__ui.twTable.getChecks(self.availtoolwidgets))
+            self.__ui.twTable.getChecks(self.availtoolwidgets),
+            cls=numpyEncoder)
 
-        QtGui.QDialog.accept(self)
+        QtWidgets.QDialog.accept(self)
