@@ -32,8 +32,8 @@ import math
 import json
 import time
 import logging
-# import warnings
-from pyqtgraph.graphicsItems.ROI import ROI, LineROI, Handle
+import warnings
+from pyqtgraph.graphicsItems.ROI import ROI, LineROI, Handle, EllipseROI
 from pyqtgraph.graphicsItems.IsocurveItem import IsocurveItem
 
 _VMAJOR, _VMINOR, _VPATCH = _pg.__version__.split(".")[:3] \
@@ -42,6 +42,23 @@ _VMAJOR, _VMINOR, _VPATCH = _pg.__version__.split(".")[:3] \
 logger = logging.getLogger("lavue")
 
 
+def nozero(value, minvalue=0.1):
+    """overrides by min value if absolute value is smaller the minvalue
+
+    :param value: numeric value
+    :type value: :obj:`float`
+    :param minvalue: minimal value
+    :type minvalue: :obj:`float`
+    :returns: non-zero value
+    :rtype: :obj:`float`
+    """
+    if value >= 0 and value < minvalue:
+        return minvalue
+    elif value < 0 and value > -minvalue:
+        return -minvalue
+    return value
+
+
 class HandleWithSignals(Handle):
     """ handle with signals
 
@@ -53,9 +70,9 @@ class HandleWithSignals(Handle):
         """ constructor
 
         :param pos: position of handle
-        :type pos: [float, float]
+        :type pos: [:obj:`float`, :obj:`float`]
         :param center: center of handle
-        :type center: [float, float]
+        :type center: [:obj:`float`, :obj:`float`]
         :param parent: roi object
         :type parent: :class:`pyqtgraph.graphicsItems.ROI.ROI`
         """
@@ -86,17 +103,17 @@ class SimpleLineROI(LineROI):
         """ constructor
 
         :param pos1: start position
-        :type pos1: [float, float]
+        :type pos1: [:obj:`float`, :obj:`float`]
         :param pos2: end position
-        :type pos2: [float, float]
+        :type pos2: [:obj:`float`, :obj:`float`]
         :param args: dictionary with ROI parameters
         :type args: :obj:`dict`<:obj:`str`, :obj:`any`>
         """
         pos1 = _pg.Point(pos1)
         pos2 = _pg.Point(pos2)
         d = pos2 - pos1
-        ln = d.length()
-        ang = _pg.Point(1, 0).angle(d)
+        ln = nozero(d.length())
+        ang = -_pg.Point(1, 0).angle(d)
 
         ROI.__init__(self, pos1, size=_pg.Point(ln, width), angle=ang, **args)
         h1pos = [0, 0.0]
@@ -236,36 +253,36 @@ class DisplayExtension(QtCore.QObject):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         """
 
     def mouse_doubleclick(self, x, y, locked):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         :param locked: double click lock
-        :type locked: bool
+        :type locked: :obj:`bool`
         """
 
     def mouse_click(self, x, y):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         """
 
     def scalingLabel(self):
         """ provides scaling label
 
         :returns:  scaling label
-        :rtype: str
+        :rtype: :obj:`str`
         """
 
 
@@ -273,6 +290,8 @@ class ROIExtension(DisplayExtension):
 
     #: (:class:`pyqtgraph.QtCore.pyqtSignal`) roi coordinate changed signal
     roiCoordsChanged = QtCore.pyqtSignal()
+    #: (:class:`pyqtgraph.QtCore.pyqtSignal`) roi number changed signal
+    roiNumberChanged = QtCore.pyqtSignal(int)
 
     def __init__(self, parent=None):
         """ constructor
@@ -290,11 +309,28 @@ class ROIExtension(DisplayExtension):
         #: (:class:`pyqtgraph.QtCore.QSignalMapper`) roi region mapper
         self.__roiregionmapper = QtCore.QSignalMapper(self)
 
+        #: (:obj:`int`) current roi drown id
+        self.__drawing = -1
+        #: (:obj:`int`) current roi drown position
+        self.__drawpos = [0, 0]
+        #: (:obj:`bool`) roi on clickcurrent roi drown id
+        self.__roionclick = False
+        #: (:obj:`float`) time lock
+        self.__timelock = 0
+        #: (:obj:`float`) time to lock
+        self.__timetolock = 0.5
+
         #: (:obj:`int`) current roi id
         self.__current = 0
-        #: (:obj:`list` < [int, int, int, int] > )
+        #: (:obj:`list` <
+        #:      [ :obj:`int`, :obj:`int`, :obj:`int`, :obj:`int` ] >  or
+        #:     :obj:`list` < [:obj:`float`, :obj:`float`, :obj:`float`,
+        #:                    :obj:`float`, :obj:`float`] >)
         #: x1,y1,x2,y2 rois coordinates
         self.__coords = [[10, 10, 60, 60]]
+        #: (:obj:`list` < :obj:`str` > )
+        #: x1,y1,x2,y2 rois coordinates
+        self.__types = ["rectangle"]
         #: (:obj:`list` < (int, int, int) > ) list with roi colors
         self.__colors = []
 
@@ -304,6 +340,8 @@ class ROIExtension(DisplayExtension):
         #: (:obj:`list` <:class:`pyqtgraph.graphicsItems.ROI`>)
         #:            list of roi widgets
         self.__roi = []
+        #: (:obj:`str`) current roi type
+        self.__roitype = "rectangle"
         self.__roi.append(ROI(0, _pg.Point(50, 50)))
         self.__roi[0].addScaleHandle([1, 1], [0, 0])
         self.__roi[0].addScaleHandle([0, 0], [1, 1])
@@ -332,43 +370,74 @@ class ROIExtension(DisplayExtension):
         if parameters.rois is not None:
             self.__showROIs(parameters.rois)
             self._enabled = parameters.rois
+        if parameters.roionclick is not None:
+            self.__roionclick = parameters.roionclick
 
     def scalingLabel(self):
         """ provides scaling label
 
         :returns:  scaling label
-        :rtype: str
+        :rtype: :obj:`str`
         """
         return "intensity"
 
-    def __addROI(self, coords=None):
+    def __addROI(self, coords=None, label=None, roitype="rectangle"):
         """ adds ROIs
 
         :param coords: roi coordinates
         :type coords: :obj:`list`
                  < [:obj:`float`, :obj:`float`, :obj:`float`, :obj:`float`] >
+        :param roitype: roi type
+        :type roitype: :obj:`str`
         """
-        if not coords or not isinstance(coords, list) or len(coords) != 4:
-            pnt = 10 * len(self.__roi)
-            sz = 50
-            coords = [pnt, pnt, pnt + sz, pnt + sz]
-            spnt = _pg.Point(sz, sz)
-        else:
-            if not self._mainwidget.transformations()[0]:
-                pnt = _pg.Point(coords[0], coords[1])
-                spnt = _pg.Point(coords[2] - coords[0], coords[3] - coords[1])
+        if roitype == "ellipse":
+            angle = 0.0
+            if not coords or not isinstance(coords, list) or len(coords) < 4:
+                pnt = 50. + 10. * len(self.__roi)
+                sz = 50.
+                coords = [pnt, pnt, sz, sz, angle]
+                spnt = _pg.Point(sz, sz)
             else:
-                pnt = _pg.Point(coords[1], coords[0])
-                spnt = _pg.Point(coords[3] - coords[1], coords[2] - coords[0])
-        self.__roi.append(ROI(pnt, spnt))
-        self.__roi[-1].addScaleHandle([1, 1], [0, 0])
-        self.__roi[-1].addScaleHandle([0, 0], [1, 1])
-        text = _pg.TextItem("%s." % len(self.__roi), anchor=(1, 1))
+                if not self._mainwidget.transformations()[0]:
+                    pnt = _pg.Point(coords[0], coords[1])
+                    spnt = _pg.Point(nozero(coords[2]), nozero(coords[3]))
+                    if len(coords) > 4:
+                        angle = coords[4]
+                else:
+                    pnt = _pg.Point(coords[1], coords[0])
+                    spnt = _pg.Point(nozero(coords[3]), nozero(coords[2]))
+                    if len(coords) > 4:
+                        angle = -coords[4]
+
+            self.__roi.append(EllipseROI(pnt, spnt, angle=angle))
+        else:
+            if not coords or not isinstance(coords, list) or len(coords) != 4:
+                pnt = 10 * len(self.__roi)
+                sz = 50
+                coords = [pnt, pnt, pnt + sz, pnt + sz]
+                spnt = _pg.Point(sz, sz)
+            else:
+                if not self._mainwidget.transformations()[0]:
+                    pnt = _pg.Point(coords[0], coords[1])
+                    spnt = _pg.Point(coords[2] - coords[0],
+                                     coords[3] - coords[1])
+                else:
+                    pnt = _pg.Point(coords[1], coords[0])
+                    spnt = _pg.Point(coords[3] - coords[1],
+                                     coords[2] - coords[0])
+            self.__roi.append(ROI(pnt, spnt))
+            self.__roi[-1].addScaleHandle([1, 1], [0, 0])
+            self.__roi[-1].addScaleHandle([0, 0], [1, 1])
+        if label:
+            text = _pg.TextItem("%s" % label, anchor=(1, 1))
+        else:
+            text = _pg.TextItem("%s." % len(self.__roi), anchor=(1, 1))
         text.setParentItem(self.__roi[-1])
         self.__roitext.append(text)
         self._mainwidget.viewbox().addItem(self.__roi[-1])
 
         self.__coords.append(coords)
+        self.__types.append(roitype)
         self.setColors()
 
     def __removeROI(self):
@@ -380,6 +449,7 @@ class ROIExtension(DisplayExtension):
         roitext.hide()
         self._mainwidget.viewbox().removeItem(roi)
         self.__coords.pop()
+        self.__types.pop()
 
     def _getROI(self, rid=-1):
         """ get the given or the last ROI
@@ -405,27 +475,59 @@ class ROIExtension(DisplayExtension):
             for roi in self.__roi:
                 roi.hide()
 
-    def __addROICoords(self, coords):
+    def __addROICoords(self, coords, types):
         """ adds ROI coorinates
 
         :param coords: roi coordinates
         :type coords: :obj:`list`
                 < [:obj:`float`, :obj:`float`, :obj:`float`, :obj:`float`] >
+        :param types: roi types
+        :type types: :obj:`list` < [:obj:`str`] >
         """
         if coords:
             for i, crd in enumerate(self.__roi):
                 if i < len(coords):
+                    roitype = "rectangle"
+                    if types and i < len(types):
+                        roitype = types[i]
+                    roiremove = False
+                    if isinstance(crd, EllipseROI) and roitype != "ellipse":
+                        roiremove = True
+                    if not isinstance(crd, EllipseROI) and \
+                            roitype == "ellipse":
+                        roiremove = True
+                    if roiremove:
+                        while self._getROI(max(i, 0)) is not None:
+                            self.__currentroimapper.removeMappings(
+                                self._getROI())
+                            self.__roiregionmapper.removeMappings(
+                                self._getROI())
+                            self.__removeROI()
+                        break
                     self.__coords[i] = coords[i]
-                    if not self._mainwidget.transformations()[0]:
-                        crd.setPos([coords[i][0], coords[i][1]])
-                        crd.setSize(
-                            [coords[i][2] - coords[i][0],
-                             coords[i][3] - coords[i][1]])
+                    while len(self.__types) < i + 1:
+                        self.__types.append("rectangle")
+                    self.__types[i] = roitype
+                    if roitype == "ellipse":
+                        if not self._mainwidget.transformations()[0]:
+                            crd.setPos([coords[i][0], coords[i][1]])
+                            crd.setSize([coords[i][2], coords[i][3]])
+                            crd.setAngle(coords[i][4])
+                        else:
+                            crd.setPos([coords[i][1], coords[i][0]])
+                            crd.setSize([coords[i][3], coords[i][2]])
+                            crd.setAngle(-coords[i][4])
                     else:
-                        crd.setPos([coords[i][1], coords[i][0]])
-                        crd.setSize(
-                            [coords[i][3] - coords[i][1],
-                             coords[i][2] - coords[i][0]])
+                        if not self._mainwidget.transformations()[0]:
+                            crd.setPos([coords[i][0], coords[i][1]])
+                            crd.setSize(
+                                [coords[i][2] - coords[i][0],
+                                 coords[i][3] - coords[i][1]])
+                        else:
+                            crd.setPos([coords[i][1], coords[i][0]])
+                            crd.setSize(
+                                [coords[i][3] - coords[i][1],
+                                 coords[i][2] - coords[i][0]])
 
     def __calcROIsum(self, rid):
         """calculates the current roi sum
@@ -433,47 +535,58 @@ class ROIExtension(DisplayExtension):
         :param rid: roi id
         :type rid: :obj:`int`
         :returns: sum roi value, roi id
-        :rtype: (float, int)
+        :rtype: (:obj:`float`, :obj:`int`)
         """
         if rid >= 0:
             image = self._mainwidget.rawData()
             if image is not None:
                 if self._enabled:
                     if rid >= 0:
-                        roicoords = self.__coords
-                        if not self._mainwidget.transformations()[0]:
-                            rcrds = list(roicoords[rid])
-                            if self._mainwidget.rangeWindowEnabled():
-                                tx, ty = self._mainwidget.descaledxy(
-                                    rcrds[0], rcrds[1], useraxes=False)
-                                if tx is not None:
-                                    tx2, ty2 = self._mainwidget.descaledxy(
-                                        rcrds[2], rcrds[3], useraxes=False)
-                                    rcrds = [tx, ty, tx2, ty2]
+                        roitype = "rectangle"
+                        if self.__types and rid < len(self.__types):
+                            roitype = self.__types[rid]
+                        if roitype == "rectangle":
+                            roicoords = self.__coords
+                            if not self._mainwidget.transformations()[0]:
+                                rcrds = list(roicoords[rid])
+                                if self._mainwidget.rangeWindowEnabled():
+                                    tx, ty = self._mainwidget.descaledxy(
+                                        rcrds[0], rcrds[1], useraxes=False)
+                                    if tx is not None:
+                                        tx2, ty2 = self._mainwidget.descaledxy(
+                                            rcrds[2], rcrds[3], useraxes=False)
+                                        rcrds = [tx, ty, tx2, ty2]
+                            else:
+                                rc = roicoords[rid]
+                                rcrds = [rc[1], rc[0], rc[3], rc[2]]
+                                if self._mainwidget.rangeWindowEnabled():
+                                    ty, tx = self._mainwidget.descaledxy(
+                                        rcrds[1], rcrds[0], useraxes=False)
+                                    if ty is not None:
+                                        ty2, tx2 = self._mainwidget.descaledxy(
+                                            rcrds[3], rcrds[2], useraxes=False)
+                                        rcrds = [tx, ty, tx2, ty2]
+                            for i in [0, 2]:
+                                if rcrds[i] > image.shape[0]:
+                                    rcrds[i] = image.shape[0]
+                                elif rcrds[i] < -i // 2:
+                                    rcrds[i] = -i // 2
+                            for i in [1, 3]:
+                                if rcrds[i] > image.shape[1]:
+                                    rcrds[i] = image.shape[1]
+                                elif rcrds[i] < - (i - 1) // 2:
+                                    rcrds[i] = - (i - 1) // 2
+                            roival = np.nansum(image[
+                                int(rcrds[0]):(int(rcrds[2]) + 1),
+                                int(rcrds[1]):(int(rcrds[3]) + 1)
+                            ])
                         else:
-                            rc = roicoords[rid]
-                            rcrds = [rc[1], rc[0], rc[3], rc[2]]
-                            if self._mainwidget.rangeWindowEnabled():
-                                ty, tx = self._mainwidget.descaledxy(
-                                    rcrds[1], rcrds[0], useraxes=False)
-                                if ty is not None:
-                                    ty2, tx2 = self._mainwidget.descaledxy(
-                                        rcrds[3], rcrds[2], useraxes=False)
-                                    rcrds = [tx, ty, tx2, ty2]
-                        for i in [0, 2]:
-                            if rcrds[i] > image.shape[0]:
-                                rcrds[i] = image.shape[0]
-                            elif rcrds[i] < -i // 2:
-                                rcrds[i] = -i // 2
-                        for i in [1, 3]:
-                            if rcrds[i] > image.shape[1]:
-                                rcrds[i] = image.shape[1]
-                            elif rcrds[i] < - (i - 1) // 2:
-                                rcrds[i] = - (i - 1) // 2
-                        roival = np.nansum(image[
-                            int(rcrds[0]):(int(rcrds[2]) + 1),
-                            int(rcrds[1]):(int(rcrds[3]) + 1)
-                        ])
+                            roival = np.nansum(
+                                self.__roi[rid].getArrayRegion(
+                                    self._mainwidget.rawData(),
+                                    self._mainwidget.image(),
+                                    axes=(0, 1)))
+                        # print("ROI", rid, roival, roival2, rcrds)
                     else:
                         roival = 0.
                 else:
@@ -487,7 +600,7 @@ class ROIExtension(DisplayExtension):
         """calculates the current roi sum
 
         :returns: sum roi value, roi id
-        :rtype: (float, int)
+        :rtype: (:obj:`float`, :obj:`int`)
         """
         if self._enabled and self._getROI() is not None:
             rid = self.__current
@@ -498,7 +611,7 @@ class ROIExtension(DisplayExtension):
         """ calculates all roi sums
 
         :returns: sum roi value, roi id
-        :rtype: :obj:list < float >
+        :rtype: :obj:`list` < :obj:`float` >
         """
         if self._mainwidget.rawData() is None:
             return None
@@ -513,23 +626,49 @@ class ROIExtension(DisplayExtension):
             rid = self.__current
             roi = self._getROI(rid)
             if roi is not None:
+                roitype = "rectangle"
+                if self.__types and rid < len(self.__types):
+                    roitype = self.__types[rid]
                 state = roi.state
-                rcrds = [
-                    state['pos'].x(),
-                    state['pos'].y(),
-                    state['pos'].x() + state['size'].x(),
-                    state['pos'].y() + state['size'].y()]
-                if not self._mainwidget.transformations()[0]:
-                    ptx1 = int(math.floor(rcrds[0]))
-                    pty1 = int(math.floor(rcrds[1]))
-                    ptx2 = int(math.floor(rcrds[2]))
-                    pty2 = int(math.floor(rcrds[3]))
+                # print("STATE", state)
+                if roitype == "ellipse":
+                    rcrds = [
+                        state['pos'].x(),
+                        state['pos'].y(),
+                        state['size'].x(),
+                        state['size'].y(),
+                        state['angle']]
+                    if not self._mainwidget.transformations()[0]:
+                        ptx1 = float(rcrds[0])
+                        pty1 = float(rcrds[1])
+                        ptx2 = float(rcrds[2])
+                        pty2 = float(rcrds[3])
+                        angle = float(rcrds[4])
+                    else:
+                        pty1 = float(rcrds[0])
+                        ptx1 = float(rcrds[1])
+                        pty2 = float(rcrds[2])
+                        ptx2 = float(rcrds[3])
+                        angle = float(-rcrds[4])
+                    crd = [ptx1, pty1, ptx2, pty2, angle]
                 else:
-                    pty1 = int(math.floor(rcrds[0]))
-                    ptx1 = int(math.floor(rcrds[1]))
-                    pty2 = int(math.floor(rcrds[2]))
-                    ptx2 = int(math.floor(rcrds[3]))
-                crd = [ptx1, pty1, ptx2, pty2]
+                    rcrds = [
+                        state['pos'].x(),
+                        state['pos'].y(),
+                        state['pos'].x() + state['size'].x(),
+                        state['pos'].y() + state['size'].y()]
+                # ?? int for none rectangle ??
+                    if not self._mainwidget.transformations()[0]:
+                        ptx1 = int(math.floor(rcrds[0]))
+                        pty1 = int(math.floor(rcrds[1]))
+                        ptx2 = int(math.floor(rcrds[2]))
+                        pty2 = int(math.floor(rcrds[3]))
+                    else:
+                        pty1 = int(math.floor(rcrds[0]))
+                        ptx1 = int(math.floor(rcrds[1]))
+                        pty2 = int(math.floor(rcrds[2]))
+                        ptx2 = int(math.floor(rcrds[3]))
+                    crd = [ptx1, pty1, ptx2, pty2]
                 if self.__coords[rid] != crd:
                     self.__coords[rid] = crd
                     self.roiCoordsChanged.emit()
@@ -549,7 +688,26 @@ class ROIExtension(DisplayExtension):
             self.__current = rid
             self.roiCoordsChanged.emit()
 
-    def updateROIs(self, rid, coords):
+    def updateLabels(self, roilabels=[]):
+        """ update ROIs
+
+        :param roilabels: roi labels i.e. aliases
+        :type roilabels: :obj:`list` < :obj:`str` >
+        """
+        for ri, roi in enumerate(self.__roi):
+            if len(roilabels) > ri:
+                text = _pg.TextItem("%s" % roilabels[ri], anchor=(1, 1))
+            else:
+                text = _pg.TextItem("%s." % (ri + 1), anchor=(1, 1))
+            if len(self.__roitext) > ri:
+                self.__roitext[ri].setParentItem(None)
+                self.__roitext[ri] = text
+            else:
+                self.__roitext.append(text)
+            text.setParentItem(self.__roi[ri])
+        self.setColors()
+
+    def updateROIs(self, rid, coords, roilabels=[], types=None):
         """ update ROIs
 
         :param rid: roi id
@@ -557,13 +715,23 @@ class ROIExtension(DisplayExtension):
         :param coords: roi coordinates
         :type coords: :obj:`list`
                  < [:obj:`float`, :obj:`float`, :obj:`float`, :obj:`float`] >
+        :param roilabels: roi labels i.e. aliases
+        :type roilabels: :obj:`list`< :obj:`str` >
+        :param types: roi types
+        :type types: :obj:`list` < [:obj:`str`] >
         """
-        self.__addROICoords(coords)
+        self.__addROICoords(coords, types)
         while rid > len(self.__roi):
+            label = None
+            if roilabels and len(roilabels) > len(self.__roi):
+                label = roilabels[len(self.__roi)]
+            crd = None
             if coords and len(coords) >= len(self.__roi):
-                self.__addROI(coords[len(self.__roi)])
-            else:
-                self.__addROI()
+                crd = coords[len(self.__roi)]
+            rtype = "rectangle"
+            if types and len(types) >= len(self.__roi):
+                rtype = types[len(self.__roi)]
+            self.__addROI(crd, label, rtype)
             self._getROI().sigHoverEvent.connect(self.__currentroimapper.map)
             self._getROI().sigRegionChanged.connect(self.__roiregionmapper.map)
             self.__currentroimapper.setMapping(
@@ -618,15 +786,43 @@ class ROIExtension(DisplayExtension):
                         self.__roitext[it].color)
         return True
 
+    def setCurrentROIType(self, rtype):
+        """ sets current ROI type
+
+        :param rtype: roi type
+        :type rtype: :obj:`str`
+        :returns: change status
+        :rtype: :obj:`bool`
+        """
+        if self.__roitype != rtype:
+            self.__roitype = rtype
+            return True
+        return False
+
+    def currentROIType(self):
+        """ current ROI type
+
+        :returns: current roi type
+        :rtype: :obj:`str`
+        """
+        return self.__roitype
+
     def roiCoords(self):
         """ provides rois coordinates
 
         :return: rois coordinates
-        :rtype: :obj:`list`
-               < [:obj:`float`, :obj:`float`, :obj:`float`, :obj:`float`] >
+        :rtype: :obj:`list` < [:obj:`list` <:obj:`float`> >
         """
         return self.__coords
 
+    def roiTypes(self):
+        """ provides rois types
+
+        :return: rois types
+        :rtype: :obj:`list` < :obj:`str` >
+        """
+        return self.__types
+
     def isROIsEnabled(self):
         """ provides flag rois enabled
 
@@ -651,12 +847,96 @@ class ROIExtension(DisplayExtension):
             size = crd.size()
             crd.setPos([pos[1], pos[0]])
             crd.setSize([size[1], size[0]])
+            crd.setAngle(-crd.angle())
+
+    def mouse_position(self, x, y):
+        """  sets vLine and hLine positions
+
+        :param x: x coordinate
+        :type x: :obj:`float`
+        :param y: y coordinate
+        :type y: :obj:`float`
+        """
+        # print("POS", x, y, self.__drawing , len(self.__roi) )
+        if self.__roionclick:
+            if self.__drawing >= 0 and len(self.__roi) > self.__drawing:
+                roi = self.__roi[self.__drawing]
+                x0, y0 = self.__drawpos
+                if self.__roitype == "ellipse":
+                    roi.setSize([nozero(x - x0), nozero(y - y0)])
+                else:
+                    roi.setSize([int(x - x0), int(y - y0)])
+
+    def mouse_doubleclick(self, x, y, locked):
+        """  sets vLine and hLine positions
+
+        :param x: x coordinate
+        :type x: :obj:`float`
+        :param y: y coordinate
+        :type y: :obj:`float`
+        :param locked: double click lock
+        :type locked: :obj:`bool`
+        """
+        # print ("DOUBLE", x,y, locked, self.__drawing)
+        if self.__roionclick:
+            if self.__drawing < 0 and \
+                    (time.time() - self.__timelock) > self.__timetolock:
+                self.__drawing = len(self.__roi)
+                coords = list(self.roiCoords())
+                types = list(self.roiTypes())
+                roitype = self.__roitype
+                if roitype == "ellipse":
+                    if not self._mainwidget.transformations()[0]:
+                        crd = [x, y, 0.0, 0.0, 0.0]
+                    else:
+                        crd = [y, x, 0.0, 0.0, 0.0]
+                else:
+                    if not self._mainwidget.transformations()[0]:
+                        crd = [int(x), int(y), int(x), int(y)]
+                    else:
+                        crd = [int(y), int(x), int(y), int(x)]
+                types.append(roitype)
+                coords.append(crd)
+                self.updateROIs(len(coords), coords, [], types)
+                self.__drawpos = [x, y]
+
+    def mouse_click(self, x, y):
+        """  sets vLine and hLine positions
+
+        :param x: x coordinate
+        :type x: :obj:`float`
+        :param y: y coordinate
+        :type y: :obj:`float`
+        """
+        # print("CLICK", x, y, self.__drawing)
+        if self.__roionclick:
+            if self.__drawing >= 0:
+                self.__timelock = time.time()
+                x0, y0 = self.__drawpos
+
+                roitype = self.__roitype
+                if roitype == "ellipse":
+                    if not self._mainwidget.transformations()[0]:
+                        crd = [x0, y0, nozero(x - x0), nozero(y - y0), 0.0]
+                    else:
+                        crd = [y0, x0, nozero(y - y0), nozero(x - x0), 0.0]
+                else:
+                    if not self._mainwidget.transformations()[0]:
+                        crd = [int(x0), int(y0), int(x), int(y)]
+                    else:
+                        crd = [int(y0), int(x0), int(y), int(x)]
+                self.__coords[-1] = crd
+                self.__types[-1] = roitype
+                self.__drawing = -1
+                self.roiNumberChanged.emit(len(self.__coords))
 
 
 class CutExtension(DisplayExtension):
 
     #: (:class:`pyqtgraph.QtCore.pyqtSignal`) cut coordinate changed signal
     cutCoordsChanged = QtCore.pyqtSignal()
+    #: (:class:`pyqtgraph.QtCore.pyqtSignal`) cut number changed signal
+    cutNumberChanged = QtCore.pyqtSignal(int)
 
     def __init__(self, parent=None):
         """ constructor
@@ -674,6 +954,15 @@ class CutExtension(DisplayExtension):
         #: (:class:`pyqtgraph.QtCore.QSignalMapper`) cut region mapper
         self.__cutregionmapper = QtCore.QSignalMapper(self)
 
+        #: (:obj:`int`) current cut drown id
+        self.__drawing = -1
+        #: (:obj:`int`) current cut drown position
+        self.__drawpos = [0, 0]
+        #: (:obj:`float`) time lock
+        self.__timelock = 0
+        #: (:obj:`float`) time to lock
+        self.__timetolock = 0.5
+
         #: (:obj:`int`) current cut id
         self.__current = 0
         #: (:obj:`list` < [int, int, int, int] > )
@@ -727,15 +1016,17 @@ class CutExtension(DisplayExtension):
                 if i < len(coords):
                     self.__coords[i] = coords[i]
                     if not self._mainwidget.transformations()[0]:
-                        crd.setPos([coords[i][0], coords[i][1]])
-                        crd.setSize(
-                            [coords[i][2] - coords[i][0],
-                             coords[i][3] - coords[i][1]])
+                        pos1 = _pg.Point([coords[i][0], coords[i][1]])
+                        pos2 = _pg.Point([coords[i][2], coords[i][3]])
                     else:
-                        crd.setPos([coords[i][1], coords[i][0]])
-                        crd.setSize(
-                            [coords[i][3] - coords[i][1],
-                             coords[i][2] - coords[i][0]])
+                        pos1 = _pg.Point([coords[i][1], coords[i][0]])
+                        pos2 = _pg.Point([coords[i][3], coords[i][2]])
+                    d = pos2 - pos1
+                    ln = nozero(d.length())
+                    ang = _pg.Point(1, 0).angle(d)
+                    crd.setPos(pos1)
+                    crd.setSize([ln, crd.size()[1]])
+                    crd.setAngle(-ang)
 
     def __addCut(self, coords=None):
         """ adds Cuts
@@ -809,8 +1100,8 @@ class CutExtension(DisplayExtension):
                     self._mainwidget.rawData(),
                     self._mainwidget.image(),
                     axes=(0, 1))
-                with np.warnings.catch_warnings():
-                    np.warnings.filterwarnings(
+                with warnings.catch_warnings():
+                    warnings.filterwarnings(
                         'ignore', r'Mean of empty slice')
                     while dt.ndim > 1:
                         dt = np.nanmean(dt, axis=1)
@@ -925,6 +1216,71 @@ class CutExtension(DisplayExtension):
                  pos[0] + math.cos(ra) * size[0]])
             crd.setAngle(270-angle)
 
+    def mouse_position(self, x, y):
+        """  sets vLine and hLine positions
+
+        :param x: x coordinate
+        :type x: :obj:`float`
+        :param y: y coordinate
+        :type y: :obj:`float`
+        """
+        # print("POS", x, y, self.__drawing, len(self.__cut))
+        if self.__drawing >= 0 and len(self.__cut) > self.__drawing:
+            pos1 = _pg.Point(self.__drawpos)
+            pos2 = _pg.Point([x, y])
+            d = pos2 - pos1
+            cut = self.__cut[self.__drawing]
+            ln = nozero(d.length())
+            ang = _pg.Point(1, 0).angle(d)
+            # cut.setPos(pos1)
+            cut.setSize([ln, cut.size()[1]])
+            cut.setAngle(-ang)
+
+    def mouse_doubleclick(self, x, y, locked):
+        """  sets vLine and hLine positions
+
+        :param x: x coordinate
+        :type x: :obj:`float`
+        :param y: y coordinate
+        :type y: :obj:`float`
+        :param locked: double click lock
+        :type locked: :obj:`bool`
+        """
+        # print("DOUBLE", x, y, self.__drawing, locked)
+        if self.__drawing < 0 and \
+                (time.time() - self.__timelock) > self.__timetolock:
+            self.__drawing = len(self.__cut)
+            coords = list(self.cutCoords())
+            if not self._mainwidget.transformations()[0]:
+                crd = [x, y, x, y, 0.00001]
+            else:
+                crd = [y, x, y, x, 0.00001]
+            coords.append(crd)
+            self.updateCuts(len(coords), coords)
+            self.__drawpos = [x, y]
+
+    def mouse_click(self, x, y):
+        """  sets vLine and hLine positions
+
+        :param x: x coordinate
+        :type x: :obj:`float`
+        :param y: y coordinate
+        :type y: :obj:`float`
+        """
+        # print("SINGLE", x, y, self.__drawing)
+        if self.__drawing >= 0:
+            self.__timelock = time.time()
+            x0, y0 = self.__drawpos
+
+            if not self._mainwidget.transformations()[0]:
+                crd = [x0, y0, x, y, 0.00001]
+            else:
+                crd = [y0, x0, y, x, 0.00001]
+            self.__coords[-1] = crd
+            self.__drawing = -1
+            self.updateCuts(len(self.__coords), self.__coords)
+            self.cutNumberChanged.emit(len(self.__coords))
+
 
 class MeshExtension(DisplayExtension):
 
@@ -993,7 +1349,7 @@ class MeshExtension(DisplayExtension):
         """ provides scaling label
 
         :returns:  scaling label
-        :rtype: str
+        :rtype: :obj:`str`
         """
         return "intensity"
 
@@ -1093,10 +1449,10 @@ class MeshExtension(DisplayExtension):
         """
         self.__addROICoords(coords)
         while rid > len(self.__roi):
+            crd = None
             if coords and len(coords) >= len(self.__roi):
-                self.__addROI(coords[len(self.__roi)])
-            else:
-                self.__addROI()
+                crd = coords[len(self.__roi)]
+            self.__addROI(crd)
             self._getROI().sigHoverEvent.connect(self.__currentroimapper.map)
             self._getROI().sigRegionChanged.connect(self.__roiregionmapper.map)
             self.__currentroimapper.setMapping(
@@ -1461,9 +1817,9 @@ class LockerExtension(DisplayExtension):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         """
         if not self.__crosshairlocked:
             now = time.time()
@@ -1491,11 +1847,11 @@ class LockerExtension(DisplayExtension):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         :param locked: double click lock
-        :type locked: bool
+        :type locked: :obj:`bool`
         """
         self.updateLocker(x, y)
 
@@ -1586,9 +1942,9 @@ class CenterExtension(DisplayExtension):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         """
         if not self.__centercoordinates:
             now = time.time()
@@ -1605,11 +1961,11 @@ class CenterExtension(DisplayExtension):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         :param locked: double click lock
-        :type locked: bool
+        :type locked: :obj:`bool`
         """
         self.updateCenter(x, y)
 
@@ -1872,9 +2228,9 @@ class MarkExtension(BaseMarkExtension):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         """
         if not self._markcoordinates:
             now = time.time()
@@ -1900,11 +2256,11 @@ class MarkExtension(BaseMarkExtension):
         """  sets vLine and hLine positions
 
         :param x: x coordinate
-        :type x: float
+        :type x: :obj:`float`
         :param y: y coordinate
-        :type y: float
+        :type y: :obj:`float`
         :param locked: double click lock
-        :type locked: bool
+        :type locked: :obj:`bool`
         """
         if not locked:
             self.updatePositionMark(x, y)
@@ -1983,7 +2339,7 @@ class MaximaExtension(DisplayExtension):
         sets maxima postions
 
         :param positionlist: [(x1, y1), ... , (xn, yn)]
-        :type positionlist: :obj:`list` < (float, float) >
+        :type positionlist: :obj:`list` < (:obj:`float`, :obj:`float`) >
         :param offset: offset of position
         :type offset: [ :obj:`float`, :obj:`float`]
         """
