@@ -26,10 +26,10 @@
 
 import weakref
 import time
-import pytz
 import datetime
 import threading
 import numpy
+import sys
 
 
 #: (:mod:`H5PYWriter`or :mod:`H5CppWriter`)
@@ -526,13 +526,20 @@ class FTFile(FTObject):
         :rtype: :obj:`str`
         """
         tzone = time.tzname[0]
+        fmt = '%Y-%m-%dT%H:%M:%S.%f%z'
         try:
-            tz = pytz.timezone(tzone)
+            if sys.version_info >= (3, 9):
+                import zoneinfo
+                tz = zoneinfo.ZoneInfo(tzone)
+                starttime = datetime.datetime.now().replace(tzinfo=tz)
+            else:
+                import pytz
+                tz = pytz.timezone(tzone)
+                starttime = tz.localize(datetime.datetime.now())
         except Exception:
             import tzlocal
             tz = tzlocal.get_localzone()
-        fmt = '%Y-%m-%dT%H:%M:%S.%f%z'
-        starttime = tz.localize(datetime.datetime.now())
+            starttime = datetime.datetime.now().replace(tzinfo=tz)
         return str(starttime.strftime(fmt))
 
     def default_field(self):
