@@ -28,10 +28,19 @@ import h5py
 import numpy as np
 import os
 import sys
+import io
 
 from . import filewriter
 # from .Types import nptype
 
+try:
+    sver = h5py.__version__.split(".", 2)
+    h5maj = int(sver[0])
+    h5min = int(sver[1])
+    h5ver = h5maj * 1000 + h5min
+except Exception:
+    h5ver = 1000
+
 
 if sys.version_info > (3,):
     unicode = str
@@ -39,6 +48,178 @@ else:
     bytes = str
 
 
+def _slice2selection(t, shape):
+    """ converts slice(s) to selection
+
+    :param t: slice tuple
+    :type t: :obj:`tuple`
+    :return shape: field shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :returns: hyperslab selection
+    :rtype: :class:`h5cpp.dataspace.Hyperslab`
+    """
+    if isinstance(t, filewriter.FTHyperslab):
+        offset = list(t.offset or [])
+        block = list(t.block or [])
+        count = list(t.count or [])
+        stride = list(t.stride or [])
+        slices = []
+        for dm, sz in enumerate(shape):
+            if len(offset) > dm:
+                if offset[dm] is None:
+                    offset[dm] = 0
+            else:
+                offset.append(0)
+            if len(block) > dm:
+                if block[dm] is None:
+                    block[dm] = 1
+            else:
+                block.append(1)
+            if len(count) > dm:
+                if count[dm] is None:
+                    count[dm] = sz
+            else:
+                count.append(sz)
+            if len(stride) > dm:
+                if stride[dm] is None:
+                    stride[dm] = 1
+            else:
+                stride.append(1)
+            slices.append(
+                h5py.MultiBlockSlice(
+                    start=offset[dm], count=count[dm],
+                    stride=stride[dm], block=block[dm]))
+        return tuple(slices)
+    return t
+
+
+def unlimited_selection(sel, shape):
+    """ checks if hyperslab is unlimited
+
+    :param sel: hyperslab selection
+    :type sel: :class:`filewriter.FTHyperslab`
+    :param shape: give shape
+    :type shape: :obj:`list`
+    :returns: if hyperslab is unlimited list
+    :rtype: :obj:`list` <:obj:`bool`>
+    """
+    res = None
+    if isinstance(sel, tuple):
+        res = []
+        for sl in sel:
+            if hasattr(sl, "stop"):
+                res.append(
+                    True if sl.stop in [unlimited()]
+                    else False)
+            elif hasattr(sl, "count"):
+                res.append(
+                    True if sl.count in [unlimited()]
+                    else False)
+            else:
+                res.append(
+                    True if sl in [unlimited()]
+                    else False)
+
+    elif hasattr(sel, "count"):
+        res = []
+        for ct in sel.count():
+            res.append(
+                True if ct in [unlimited()]
+                else False)
+    elif isinstance(sel, slice):
+        res = [True if sel.stop in [unlimited()]
+               else False]
+
+    elif sel in [unlimited()]:
+        res = [True]
+    lsh = len(shape)
+    lct = len(res)
+    ln = max(lsh, lct)
+    if res and any(t is True for t in res):
+        slices = []
+        for ii in range(ln):
+            offset = 0
+            # block = 1
+            # stride = 1
+            if ii < lsh:
+                count = shape[ii]
+            else:
+                count = 1
+            if ii < lct and res[ii]:
+                count = unlimited()
+            # print("Hyperslab %s %s %s %s" % (offset, block, count, stride))
+            slices.append(
+                slice(offset, count, None)
+                # h5py.MultiBlockSlice(
+                #     start=offset, count=count,
+                #     stride=stride, block=block)
+            )
+        return tuple(slices)
+    else:
+        return None
+
+
+def _selection2slice(t, shape):
+    """ converts selection to slice(s)
+
+    :param t: slice tuple
+    :type t: :obj:`tuple`
+    :return shape: field shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :returns: tuple of slices
+    :rtype: :obj:`tuple`<>
+    """
+    if isinstance(t, filewriter.FTHyperslab):
+        offset = list(t.offset or [])
+        block = list(t.block or [])
+        count = list(t.count or [])
+        stride = list(t.stride or [])
+        slices = []
+        for dm, sz in enumerate(shape):
+            if len(offset) > dm:
+                if offset[dm] is None:
+                    offset[dm] = 0
+            else:
+                offset.append(0)
+            if len(block) > dm:
+                if block[dm] is None:
+                    block[dm] = 1
+            else:
+                block.append(1)
+            if len(count) > dm:
+                if count[dm] is None:
+                    count[dm] = sz
+            else:
+                count.append(sz)
+            if len(stride) > dm:
+                if stride[dm] is None:
+                    stride[dm] = 1
+            else:
+                block.append(1)
+            if len(stride) > dm:
+                if stride[dm] is None:
+                    stride[dm] = 1
+            else:
+                stride.append(1)
+            if block[dm] == 1 and count[dm] == 1 and stride[dm] == 1:
+                slices.append(offset[dm])
+            elif stride[dm] == 1:
+                slices.append(slice(
+                    offset[dm], offset[dm] + block[dm] * count[dm], None))
+            elif stride[dm] != 1 and block[dm] == 1:
+                slices.append(slice(offset[dm],
+                                    offset[dm] + count[dm] * stride[dm],
+                                    stride[dm]))
+            elif stride[dm] != 1 and count[dm] == 1:
+                slices.append(slice(offset[dm],
+                                    offset[dm] + block[dm] * stride[dm],
+                                    stride[dm]))
+            else:
+                slices.append(Ellipsis)
+        return tuple(slices)
+    return t
+
+
 def nptype(dtype):
     """ converts to numpy types
 
@@ -52,6 +233,92 @@ def nptype(dtype):
     return dtype
 
 
+def is_image_file_supported():
+    """ provides if loading of image files are supported
+
+    :retruns: if loading of image files are supported
+    :rtype: :obj:`bool`
+    """
+    return h5ver >= 2009
+
+
+def is_vds_supported():
+    """ provides if VDS are supported
+
+    :retruns: if VDS are supported
+    :rtype: :obj:`bool`
+    """
+    return h5ver >= 2009
+
+
+def is_mbs_supported():
+    """ provides if MultiBlockSlice are supported
+
+    :retruns: if MultiBlockSlice are supported
+    :rtype: :obj:`bool`
+    """
+    return h5ver >= 3000
+
+
+def is_unlimited_vds_supported():
+    """ provides if unlimited vds are supported
+
+    :retruns: if unlimited vds are supported
+    :rtype: :obj:`bool`
+    """
+    return h5ver >= 3000
+
+
+def is_strings_as_bytes():
+    """ provides if string read to bytes
+
+    :retruns: if string read to bytes
+    :rtype: :obj:`bool`
+    """
+    return h5ver >= 3000
+
+
+def unlimited(parent=None):
+    """ return dataspace UNLIMITED variable for the current writer module
+
+    :param parent: parent object
+    :type parent: :class:`FTObject`
+    :returns:  dataspace UNLIMITED variable
+    :rtype: :class:`h5py.h5s.UNLIMITED`
+    """
+    try:
+        return h5py.h5s.UNLIMITED
+    except Exception:
+        return h5py.UNLIMITED
+
+
+def load_file(membuffer, filename=None, readonly=False, **pars):
+    """ load a file from memory byte buffer
+
+    :param membuffer: memory buffer
+    :type membuffer: :obj:`bytes` or :obj:`io.BytesIO`
+    :param filename: file name
+    :type filename: :obj:`str`
+    :param readonly: readonly flag
+    :type readonly: :obj:`bool`
+    :param pars: parameters
+    :type pars: :obj:`dict` < :obj:`str`, :obj:`str`>
+    :returns: file object
+    :rtype: :class:`H5PYFile`
+    """
+    if not is_image_file_supported():
+        raise Exception("Loading a file from a memory buffer not supported")
+    if not hasattr(membuffer, 'read') or not hasattr(membuffer, 'seek'):
+        if hasattr(membuffer, "tobytes"):
+            membuffer = membuffer.tobytes()
+        membuffer = io.BytesIO(membuffer)
+    if readonly:
+        fobj = h5py.File(membuffer, "r", **pars)
+    else:
+        fobj = h5py.File(membuffer, "r+", **pars)
+    return H5PYFile(fobj, filename)
+
+
 def open_file(filename, readonly=False, **pars):
     """ open the new file
 
@@ -84,9 +351,9 @@ def create_file(filename, overwrite=False, **pars):
     """
     fl = h5py.File(filename, "w" if overwrite else "w-", **pars)
     fl.attrs["file_time"] = unicode(H5PYFile.currenttime())
-    fl.attrs["HDF5_version"] = u""
+    fl.attrs["HDF5_Version"] = str(h5py.version.hdf5_version)
     fl.attrs["NX_class"] = u"NXroot"
-    fl.attrs["NeXus_version"] = u"4.3.0"
+    # fl.attrs["NeXus_version"] = u"4.3.0"
     fl.attrs["file_name"] = unicode(filename)
     fl.attrs["file_update_time"] = unicode(H5PYFile.currenttime())
     return H5PYFile(fl, filename)
@@ -126,7 +393,7 @@ def get_links(parent):
     :type parent: :class:`FTObject`
     :returns: list of link objects
     :returns: link object
-    :rtype: :obj: `list` <:class:`PNILink`>
+    :rtype: :obj: `list` <:class:`H5PYLink`>
     """
 
     return [H5PYLink(
@@ -146,6 +413,54 @@ def data_filter():
 deflate_filter = data_filter
 
 
+def target_field_view(filename, fieldpath, shape,
+                      dtype=None, maxshape=None):
+    """ create target field view for VDS
+
+    :param filename: file name
+    :type filename: :obj:`str`
+    :param fieldpath: nexus field path
+    :type fieldpath: :obj:`str`
+    :param shape: shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :param dtype: attribute type
+    :type dtype: :obj:`str`
+    :param maxshape: shape
+    :type maxshape: :obj:`list` < :obj:`int` >
+    :returns: target field view object
+    :rtype: :class:`FTTargetFieldView`
+    """
+    if not is_vds_supported():
+        raise Exception("VDS not supported")
+    if shape:
+        maxshape = maxshape or [None for _ in shape]
+    vs = h5py.VirtualSource(
+        filename, fieldpath,
+        tuple(shape or []), dtype, tuple(maxshape or []))
+    return H5PYTargetFieldView(vs, tuple(shape or []))
+
+
+def virtual_field_layout(shape, dtype, maxshape=None):
+    """ creates a virtual field layout for a VDS file
+
+    :param shape: shape
+    :type shape: :obj:`list` < :obj:`int` >
+    :param dtype: attribute type
+    :type dtype: :obj:`str`
+    :param maxshape: shape
+    :type maxshape: :obj:`list` < :obj:`int` >
+    :returns: virtual layout
+    :rtype: :class:`FTVirtualFieldLayout`
+    """
+    if not is_vds_supported():
+        raise Exception("VDS not supported")
+    maxshape = maxshape or [None for _ in shape]
+    return H5PYVirtualFieldLayout(
+        h5py.VirtualLayout(tuple(shape), dtype, tuple(maxshape or [])),
+        tuple(shape)
+    )
+
+
 class H5PYFile(filewriter.FTFile):
 
     """ file tree file
@@ -154,7 +469,7 @@ class H5PYFile(filewriter.FTFile):
     def __init__(self, h5object, filename):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param filename:  file name
         :type filename: :obj:`str`
@@ -253,7 +568,7 @@ class H5PYGroup(filewriter.FTGroup):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -374,12 +689,29 @@ class H5PYGroup(filewriter.FTGroup):
             grp.attrs["NX_class"] = unicode(nxclass)
         return H5PYGroup(grp, self)
 
+    def create_virtual_field(self, name, layout, fillvalue=None):
+        """ creates a virtual filed tres element
+
+        :param name: group name
+        :type name: :obj:`str`
+        :param layout: virual field layout
+        :type layout: :class:`H5PYFieldLayout`
+        :param fillvalue:  fill value
+        :type fillvalue: :obj:`int`
+        """
+        if not is_vds_supported():
+            raise Exception("VDS not supported")
+        return H5PYField(
+            self._h5object.create_virtual_dataset(
+                name, layout._h5object, fillvalue),
+            self)
+
     def create_field(self, name, type_code,
                      shape=None, chunk=None, dfilter=None):
-        """ open a file tree element
+        """ creates a field tree element
 
-        :param n: group name
-        :type n: :obj:`str`
+        :param name: group name
+        :type name: :obj:`str`
         :param type_code: nexus field type
         :type type_code: :obj:`str`
         :param shape: shape
@@ -391,12 +723,16 @@ class H5PYGroup(filewriter.FTGroup):
         :returns: file tree field
         :rtype: :class:`H5PYField`
         """
-        shape = shape or [1]
-        mshape = [None for _ in shape] or (None,)
         if type_code in ['string', b'string']:
             type_code = h5py.special_dtype(vlen=unicode)
-            # type_code = h5py.special_dtype(vlen=unicode)
             # type_code = h5py.special_dtype(vlen=bytes)
+        if type_code == h5py.special_dtype(vlen=unicode) and \
+           shape is None and chunk is None:
+            return H5PYField(
+                self._h5object.create_dataset(name, (), type_code), self)
+
+        shape = shape or [1]
+        mshape = [None for _ in shape] or (None,)
         if dfilter:
             if dfilter.filterid == 1:
                 f = H5PYField(
@@ -465,7 +801,7 @@ class H5PYGroup(filewriter.FTGroup):
     def names(self):
         """ read the child names
 
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`list` <`str`>
         """
         return list(self._h5object.keys())
@@ -494,13 +830,13 @@ class H5PYGroup(filewriter.FTGroup):
 
 class H5PYField(filewriter.FTField):
 
-    """ file tree file
+    """ file writer field
     """
 
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -553,25 +889,34 @@ class H5PYField(filewriter.FTField):
         :type dim: :obj:`int`
         """
         shape = list(self._h5object.shape)
-        shape[dim] += ext
-        return self._h5object.resize(shape)
+        if shape:
+            shape[dim] += ext
+            return self._h5object.resize(shape)
+        else:
+            return self._h5object
 
     def read(self):
         """ read the field value
 
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`any`
         """
         fl = self._h5object[...]
         if hasattr(fl, "decode") and not isinstance(fl, unicode):
-            return fl.decode(encoding="utf-8")
-        else:
-            return fl
+            fl = fl.decode(encoding="utf-8")
+        if is_strings_as_bytes() and hasattr(fl, "astype") and \
+           self.dtype in ['string', b'string']:
+            try:
+                fl = fl.astype('str')
+            except Exception:
+                # print(str(e))
+                pass
+        return fl
 
     def write(self, o):
         """ write the field value
 
-        :param o: pni object
+        :param o: h5 object
         :type o: :obj:`any`
         """
         self._h5object[...] = o
@@ -581,7 +926,7 @@ class H5PYField(filewriter.FTField):
 
         :param t: slice tuple
         :type t: :obj:`tuple`
-        :param o: pni object
+        :param o: h5 object
         :type o: :obj:`any`
         """
         if isinstance(o, np.ndarray):
@@ -603,14 +948,20 @@ class H5PYField(filewriter.FTField):
 
         :param t: slice tuple
         :type t: :obj:`tuple`
-        :returns: pni object
+        :returns: h5 object
         :rtype: :obj:`any`
         """
         fl = self._h5object.__getitem__(t)
         if hasattr(fl, "decode") and not isinstance(fl, unicode):
-            return fl.decode(encoding="utf-8")
-        else:
-            return fl
+            fl = fl.decode(encoding="utf-8")
+        if is_strings_as_bytes() and hasattr(fl, "astype") and \
+           self.dtype in ['string', b'string']:
+            try:
+                fl = fl.astype('str')
+            except Exception:
+                # print(str(e))
+                pass
+        return fl
 
     @property
     def is_valid(self):
@@ -670,7 +1021,7 @@ class H5PYLink(filewriter.FTLink):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -731,6 +1082,13 @@ class H5PYLink(filewriter.FTLink):
 
     @classmethod
     def getfilename(cls, obj):
+        """ provides a filename from h5 node
+
+        :param obj: h5 node
+        :type obj: :class:`FTObject`
+        :returns: file name
+        :rtype: :obj:`str`
+        """
         filename = ""
         while not filename:
             par = obj.parent
@@ -780,6 +1138,88 @@ class H5PYDataFilter(filewriter.FTDataFilter):
     """
 
 
+class H5PYVirtualFieldLayout(filewriter.FTVirtualFieldLayout):
+
+    """ virtual field layout """
+
+    def __init__(self, h5object, shape):
+        """ constructor
+
+        :param h5object: h5 object
+        :type h5object: :obj:`any`
+        :param shape: shape
+        :type shape: :obj:`list` < :obj:`int` >
+        """
+        filewriter.FTVirtualFieldLayout.__init__(self, h5object)
+        #: (:obj:`list` < :obj:`int` >) shape
+        self.shape = shape
+
+    def __setitem__(self, key, source):
+        """ add target field to layout
+
+        :param key: slide
+        :type key: :obj:`tuple`
+        :param source: target field view
+        :type source: :class:`H5PYTargetFieldView`
+        """
+        #: (:obj:`list` < :obj:`int` >) shape
+        self._h5object.__setitem__(key, source._h5object)
+
+    def add(self, key, source, sourcekey=None, shape=None):
+        """ add target field to layout
+
+        :param key: slide
+        :type key: :obj:`tuple`
+        :param source: target field view
+        :type source: :class:`H5PYTargetFieldView`
+        :param sourcekey: slide or selection
+        :type sourcekey: :obj:`tuple`
+        :param shape: target shape in the layout
+        :type shape: :obj:`tuple`
+        """
+        if shape is None:
+            shape = list(source.shape or [])
+            if hasattr(key, "__len__"):
+                size = len(key)
+                while len(shape) < size:
+                    shape.insert(0, 1)
+        # # if is_mbs_supported():
+        #     key = _slice2selection(key, tuple(shape))
+        # else:
+        key = _selection2slice(key, tuple(shape))
+        if sourcekey is not None and sourcekey != filewriter.FTHyperslab():
+            if is_mbs_supported():
+                sourcekey = _slice2selection(sourcekey, source.shape)
+            else:
+                sourcekey = _selection2slice(sourcekey, source.shape)
+            self._h5object.__setitem__(key, source._h5object[sourcekey])
+        elif key is not None:
+            usel = unlimited_selection(key, shape)
+            if usel is not None:
+                self._h5object[key] = source._h5object[usel]
+            else:
+                self._h5object.__setitem__(key, source._h5object)
+        else:
+            self._h5object.__setitem__(key, source._h5object)
+
+
+class H5PYTargetFieldView(filewriter.FTTargetFieldView):
+
+    """ target field view for VDS """
+
+    def __init__(self, h5object, shape):
+        """ constructor
+
+        :param h5object: h5 object
+        :type h5object: :obj:`any`
+        :param shape: shape
+        :type shape: :obj:`list` < :obj:`int` >
+        """
+        filewriter.FTTargetFieldView.__init__(self, h5object)
+        #: (:obj:`list` < :obj:`int` >) shape
+        self.shape = shape
+
+
 class H5PYDeflate(H5PYDataFilter):
     pass
 
@@ -792,7 +1232,7 @@ class H5PYAttributeManager(filewriter.FTAttributeManager):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -830,12 +1270,17 @@ class H5PYAttributeManager(filewriter.FTAttributeManager):
                 shape = tuple(shape)
             if dtype in ['string', b'string']:
                 dtype = h5py.special_dtype(vlen=unicode)
+                if is_strings_as_bytes():
+                    etype = 'str'
+                else:
+                    etype = dtype
                 self._h5object.create(
-                    name, np.empty(shape, dtype=dtype),
-                    shape, nptype(dtype))
+                    name, np.empty(shape, dtype=etype),
+                    shape=shape, dtype=nptype(dtype))
             else:
                 self._h5object.create(
-                    name, np.zeros(shape, dtype=dtype), shape, dtype)
+                    name, np.zeros(shape, dtype=dtype),
+                    shape, dtype)
         else:
             if dtype in ['string', b'string']:
                 dtype = h5py.special_dtype(vlen=unicode)
@@ -908,6 +1353,14 @@ class H5PYAttributeManager(filewriter.FTAttributeManager):
         """
         return H5PYAttribute((self._h5object, name), self.parent)
 
+    def names(self):
+        """ key values
+
+        :returns: attribute names
+        :rtype: :obj:`list` <:obj:`str`>
+        """
+        return self._h5object.keys()
+
     def reopen(self):
         """ reopen field
         """
@@ -937,7 +1390,7 @@ class H5PYAttribute(filewriter.FTAttribute):
     def __init__(self, h5object, tparent=None):
         """ constructor
 
-        :param h5object: pni object
+        :param h5object: h5 object
         :type h5object: :obj:`any`
         :param tparent: treee parent
         :type tparent: :obj:`FTObject`
@@ -1098,9 +1551,9 @@ class H5PYAttribute(filewriter.FTAttribute):
         :rtype: :obj:`list` < :obj:`int` >
         """
         if hasattr(self._h5object[0][self.name], "shape"):
-            return self._h5object[0][self.name].shape or (1,)
+            return self._h5object[0][self.name].shape
         else:
-            return (1,)
+            return ()
 
     def reopen(self):
         """ reopen attribute
