@@ -28,15 +28,12 @@
 #: (:obj:`dict` < :obj:`str`, :obj:`list` <:obj:`str`> >)
 #:  server dictionary
 HIDRASERVERLIST = {
-    "p03": ['haspp03pilatus.desy.de'],
     "p08": ['haspp08pil100.desy.de',
             'haspp08perk01.desy.de'],
     "p09": ['haspp09pilatus.desy.de'],
     "p10": ['haspp10pilatus.desy.de'],
     "p11": ['haspp11pilatus.desy.de'],
     "pool": [
-        'haspilatus300k.desy.de',
-        'haspilatus1m.desy.de',
         'haspilatus100k.desy.de'
     ]
 }
