@@ -27,9 +27,16 @@
 
 
 import pyqtgraph as _pg
-from pyqtgraph import QtCore, QtGui
+from pyqtgraph import QtCore
 import numpy as np
 import logging
+import warnings
+
+try:
+    from pyqtgraph import QtWidgets
+except Exception:
+    from pyqtgraph import QtGui as QtWidgets
+
 
 from .external.pyqtgraph_0_12 import (
     histogram__init__, histogram_paint, histogram_setHistogramRange,
@@ -108,7 +115,7 @@ class HistogramHLUTWidget(_pg.widgets.GraphicsView.GraphicsView):
         self.item = HistogramHLUTItem(bins, step, *args, **kargs)
         self.setCentralItem(self.item)
         self.setSizePolicy(
-            QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Expanding)
+            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
         self.setMinimumWidth(95)
         self.setMinimumHeight(95)
 
@@ -173,10 +180,10 @@ class GradientEditorItemWS(
         self.__skipupdate = False
         #: (:obj:`str`) color gradient name
         self.name = "highcontrast"
-        #: (:class:`pyqtgrath.QtGui.QAction`) save gradient action
-        self.saveAction = QtGui.QAction('Save ...', self)
-        #: (:obj:`pyqtgrath.QtGui.QAction`) remove gradient action
-        self.removeAction = QtGui.QAction('Remove', self)
+        #: (:class:`pyqtgrath.QtWidgets.QAction`) save gradient action
+        self.saveAction = QtWidgets.QAction('Save ...', self)
+        #: (:obj:`pyqtgrath.QtWidgets.QAction`) remove gradient action
+        self.removeAction = QtWidgets.QAction('Remove', self)
 
     def addMenuActions(self):
         """ add save/remove actions
@@ -351,8 +358,8 @@ class HistogramHLUTItem(_pg.HistogramLUTItem):
         #: (:obj: `bool`) rgb flag
         self.__rgb = False
 
-        #: (:class:`PyQt5.QtGui.QGraphicsGridLayout`) grid layout
-        self.layout = QtGui.QGraphicsGridLayout()
+        #: (:class:`PyQt5.QtWidgets.QGraphicsGridLayout`) grid layout
+        self.layout = QtWidgets.QGraphicsGridLayout()
         self.setLayout(self.layout)
         self.layout.setContentsMargins(1, 1, 1, 1)
         self.layout.setSpacing(0)
@@ -632,8 +639,12 @@ class HistogramHLUTItem(_pg.HistogramLUTItem):
         hy = None
         if self.autolevelfactor is not None:
             try:
-                hx, hy = self.__imageItem().getHistogram(
-                    step=self.__step, bins=self.__bins)
+                with warnings.catch_warnings():
+                    warnings.filterwarnings(
+                        'ignore',
+                        r'All-NaN slice encountered')
+                    hx, hy = self.__imageItem().getHistogram(
+                        step=self.__step, bins=self.__bins)
             except Exception as e:
                 logger.warning(str(e))
                 # print(str(e))
@@ -662,8 +673,12 @@ class HistogramHLUTItem(_pg.HistogramLUTItem):
         """
         channels = []
         if ch is None:
-            channels = self.__imageItem().getHistogram(perChannel=True)
-        if ch[0] is None:
+            with warnings.catch_warnings():
+                warnings.filterwarnings(
+                    'ignore',
+                    r'All-NaN slice encountered')
+                channels = self.__imageItem().getHistogram(perChannel=True)
+        if not ch or ch[0] is None:
             return
         autofactor = False
         if self.autolevelfactor is not None:
@@ -717,8 +732,12 @@ class HistogramHLUTItem(_pg.HistogramLUTItem):
                 # _pg.graphicsItems.HistogramLUTItem.HistogramLUTItem.\
                 #     imageChanged(
                 #         self, autoLevel=autoLevel, autoRange=autoRange)
-                h = self.__imageItem().getHistogram(
-                    step=self.__step, bins=self.__bins)
+                with warnings.catch_warnings():
+                    warnings.filterwarnings(
+                        'ignore',
+                        r'All-NaN slice encountered')
+                    h = self.__imageItem().getHistogram(
+                        step=self.__step, bins=self.__bins)
                 if h[0] is None:
                     return
                 self.plot.setData(*h)
@@ -732,8 +751,12 @@ class HistogramHLUTItem(_pg.HistogramLUTItem):
         else:
             # plot one histogram for each channel
             self.plots[0].setVisible(False)
-            ch = self.__imageItem().getHistogram(perChannel=True)
-            if ch[0] is None:
+            with warnings.catch_warnings():
+                warnings.filterwarnings(
+                    'ignore',
+                    r'All-NaN slice encountered')
+                ch = self.__imageItem().getHistogram(perChannel=True)
+            if not ch or ch[0] is None:
                 return
             for i in range(1, 5):
                 if len(ch) >= i:
