@@ -31,6 +31,13 @@ from pyqtgraph import QtCore, QtGui
 import math
 import types
 import logging
+import json
+
+try:
+    from pyqtgraph import QtWidgets
+except Exception:
+    from pyqtgraph import QtGui as QtWidgets
+
 
 from . import axesDialog
 from . import memoExportDialog
@@ -156,7 +163,7 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         :type parent: :class:`pyqtgraph.QtCore.QObject`
         """
         _pg.GraphicsLayoutWidget.__init__(self, parent)
-        #: (:class:`PyQt5.QtGui.QLayout`) the main layout
+        #: (:class:`PyQt5.QtWidgets.QLayout`) the main layout
         self.__layout = self.ci
 
         #: (:class:`lavuelib.imageDisplayWidget.AxesParameters`)
@@ -183,6 +190,8 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
 
         #: (:obj:`list` < :class:`pyqtgraph.ImageItem` >) image item list
         self.__images = [SafeImageItem(), SafeImageItem(), SafeImageItem()]
+        #: (:obj:`list` < :class:`pyqtgraph.ImageItem` >) mask image item list
+        self.__maskImages = [SafeImageItem(), SafeImageItem(), SafeImageItem()]
         #: (:class:`pyqtgraph.ImageItem`) image item
         self.__image = self.__images[0]
 
@@ -194,6 +203,12 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         self.__viewbox.addItem(self.__images[2])
         self.__images[1].hide()
         self.__images[2].hide()
+        self.__viewbox.addItem(self.__maskImages[0])
+        self.__viewbox.addItem(self.__maskImages[1])
+        self.__viewbox.addItem(self.__maskImages[2])
+        self.__maskImages[0].hide()
+        self.__maskImages[1].hide()
+        self.__maskImages[2].hide()
         #: (:obj:`float`) current floar x-position
         self.__xfdata = 0
         #: (:obj:`float`) current floar y-position
@@ -214,6 +229,8 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         self.__doubleclicklock = False
         #: (:obj:`bool`) rgb on flag
         self.__rgb = False
+        #: (:obj:`tuple` <:obj:`int`>) mask color
+        self.__overflowcolor = (255, 255, 255)
         #: (:obj:`bool`) gradient colors flag
         self.__gradientcolors = False
         #: (:obj:`str`) levelmode
@@ -221,21 +238,33 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         #: (:obj:`dict` < :obj:`str`, :obj:`DisplayExtension` >)
         #          extension dictionary with name keys
         self.__extensions = {}
-        #: (:class:`PyQt5.QtGui.QAction`) set aspect ration locked action
-        self.__setaspectlocked = QtGui.QAction(
+        #: (:class:`PyQt5.QtWidgets.QAction`) set aspect ration locked action
+        self.__setaspectlocked = QtWidgets.QAction(
             "Set Aspect Locked", self.__viewbox.menu)
         self.__setaspectlocked.setCheckable(True)
         if _PQGVER < 1009:
             self.__viewbox.menu.axes.insert(0, self.__setaspectlocked)
         self.__viewbox.menu.addAction(self.__setaspectlocked)
 
-        #: (:class:`PyQt5.QtGui.QAction`) view one to one pixel action
-        self.__viewonetoone = QtGui.QAction(
+        #: (:class:`PyQt5.QtWidgets.QAction`) view one to one pixel action
+        self.__viewonetoone = QtWidgets.QAction(
             "View 1:1 pixels", self.__viewbox.menu)
         self.__viewonetoone.triggered.connect(self._oneToOneRange)
         if _PQGVER < 1009:
             self.__viewbox.menu.axes.insert(0, self.__viewonetoone)
         self.__viewbox.menu.addAction(self.__viewonetoone)
+        self.__viewbox.menu.ctrl[0].invertCheck.hide()
+        self.__viewbox.menu.ctrl[0].mouseCheck.hide()
+        self.__viewbox.menu.ctrl[0].linkCombo.hide()
+        self.__viewbox.menu.ctrl[0].autoPanCheck.hide()
+        self.__viewbox.menu.ctrl[0].visibleOnlyCheck.hide()
+        self.__viewbox.menu.ctrl[0].label.hide()
+        self.__viewbox.menu.ctrl[1].invertCheck.hide()
+        self.__viewbox.menu.ctrl[1].mouseCheck.hide()
+        self.__viewbox.menu.ctrl[1].linkCombo.hide()
+        self.__viewbox.menu.ctrl[1].autoPanCheck.hide()
+        self.__viewbox.menu.ctrl[1].visibleOnlyCheck.hide()
+        self.__viewbox.menu.ctrl[1].label.hide()
 
         #: (:class:`pyqtgraph.AxisItem`) left axis
         self.__leftaxis = _pg.AxisItem('left')
@@ -261,6 +290,18 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         self.sceneObj.rawdata = None
         self.setCursor(QtGui.QCursor(QtCore.Qt.CrossCursor))
 
+    def setOverflowColor(self, color):
+        """ sets item color
+
+        :param color: json list of overflow color
+        :type color: :obj:`str`
+        """
+        try:
+            # print("COL", color)
+            self.__overflowcolor = tuple(json.loads(color))
+        except Exception:
+            self.__overflowcolor = (255, 255, 255)
+
     def viewbox(self):
         """provides viewbox
         :rtype: :class:`pyqtgraph.ViewBox`
@@ -410,6 +451,8 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
             axes.scale = scale
         for image in self.__images:
             image.resetTransform()
+        for image in self.__maskImages:
+            image.resetTransform()
         if axes.scale is not None and anyupdate:
             if not self.__transformations.transpose:
                 self._imagescale(*axes.scale)
@@ -418,18 +461,24 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                     axes.scale[1], axes.scale[0])
         else:
             self._imagescale(1, 1)
-        for image in self.__images:
+        for iid, image in enumerate(self.__images):
             if axes.position is not None and anyupdate:
                 if self.__transformations.orgtranspose and wrenabled:
                     image.setPos(
                         axes.position[1], axes.position[0])
+                    self.__maskImages[iid].setPos(
+                        axes.position[1], axes.position[0])
                 elif not self.__transformations.transpose:
                     image.setPos(*axes.position)
+                    self.__maskImages[iid].setPos(*axes.position)
                 else:
                     image.setPos(
                         axes.position[1], axes.position[0])
+                    self.__maskImages[iid].setPos(
+                        axes.position[1], axes.position[0])
             else:
                 image.setPos(0, 0)
+                self.__maskImages[iid].setPos(0, 0)
         if self.sceneObj.rawdata is not None and update:
             self.autoRange()
 
@@ -441,13 +490,15 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         :param y: y pixel coordinate
         :type y: float
         """
-        for image in self.__images:
+        for iid, image in enumerate(self.__images):
             try:
                 tr = image.transform()
                 tr.scale(x, y)
                 image.setTransform(tr)
+                self.__maskImages[iid].setTransform(tr)
             except Exception:
                 image.scale(x, y)
+                self.__maskImages[iid].scale(x, y)
 
     def setToolScale(self, position=None, scale=None):
         """ set axes scales
@@ -484,7 +535,8 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                 if self.__axes.position is None \
                 else self.__axes.position
             scale = self.__axes.scale
-        elif self.__wraxes.scale is not None and self.__wraxes.enabled is True:
+        elif self.__wraxes.scale is not None and \
+                self.__wraxes.enabled is True:
             position = [0, 0] \
                 if self.__wraxes.position is None \
                 else self.__wraxes.position
@@ -505,26 +557,30 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         else:
             axes = self.__wraxes
 
-        for image in self.__images:
+        for iid, image in enumerate(self.__images):
             if axes.scale is not None or axes.position is not None:
                 image.resetTransform()
+                self.__maskImages[iid].resetTransform()
         if axes.scale is not None:
             self._imagescale(1, 1)
-        for image in self.__images:
+        for iid, image in enumerate(self.__images):
             if axes.position is not None:
                 image.setPos(0, 0)
+                self.__maskImages[iid].setPos(0, 0)
         if axes.scale is not None or axes.position is not None:
             if self.sceneObj.rawdata is not None:
                 self.autoRange()
             self.__setLabels()
 
-    def updateImage(self, img=None, rawimg=None):
+    def updateImage(self, img=None, rawimg=None, maskimg=None):
         """ updates the image to display
 
         :param img: 2d image array
         :type img: :class:`numpy.ndarray`
         :param rawimg: 2d raw image array
         :type rawimg: :class:`numpy.ndarray`
+        :param maskarray: 2d color mask image array
+        :type maskarray: :class:`numpy.ndarray`
         """
         try:
             if img is not None and len(img.shape) == 3:
@@ -536,10 +592,13 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                         if iid < img.shape[2]:
                             if np.isnan(img[:, :, iid]).all():
                                 image.hide()
+                                self.__maskImages[iid].hide()
                             else:
                                 image.show()
                                 if self.__channellevels and \
                                    self.levelMode() != 'mono':
+                                    # print("SetImg: 1 -  RGB with RGB levels "
+                                    #       "(3 mono grad  separate images)")
                                     image.setImage(
                                         img[:, :, iid],
                                         levels=self.__channellevels[iid],
@@ -547,15 +606,35 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                                 elif self.__displaylevels[0] is not None \
                                         and self.__displaylevels[1] \
                                         is not None:
+                                    # print("SetImg: 1b - RGB with MONO levels"
+                                    #       " (3 mono grad separate images)")
                                     image.setImage(
                                         img[:, :, iid],
                                         levels=self.__displaylevels,
                                         autoLevels=False)
                                 else:
+                                    # print("SetImg: 1c - RGB with w/olevels "
+                                    #       "(3 mono grad separate images)")
                                     image.setImage(
                                         img[:, :, iid],
                                         # levels=[[0,255], [0, 255], [0, 255]],
                                         autoLevels=False)
+                                if maskimg is not None:
+                                    # print(maskimg)
+                                    if self.__overflowcolor:
+                                        self.__maskImages[iid].show()
+                                        # print(self.__overflowcolor)
+                                        self.__maskImages[iid].setImage(
+                                            np.array([
+                                                maskimg[:, :, iid].T
+                                                * self.__overflowcolor[0],
+                                                maskimg[:, :, iid].T
+                                                * self.__overflowcolor[1],
+                                                maskimg[:, :, iid].T
+                                                * self.__overflowcolor[2]
+                                            ]).T,
+                                            levels=[0, 255],
+                                            autoLevels=False)
                 else:
                     self._hideimages()
                     self.__image.show()
@@ -564,6 +643,7 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                     if img.dtype.kind == 'f' and np.isnan(img.min()):
                         img = np.nan_to_num(img)
                     if self.__channellevels and self.levelMode() != 'mono':
+                        # print("SetImg: 2a -  RGB with RGB levels")
                         self.__image.setImage(
                             img,
                             # lut=None,
@@ -571,22 +651,43 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                             autoLevels=False)
                     elif self.__displaylevels[0] is not None \
                             and self.__displaylevels[1] is not None:
+                        # print("SetImg: 2b -  RGB with MONO levels ")
                         self.__image.setImage(
                             img,
                             # lut=None,
                             levels=self.__displaylevels,
                             autoLevels=False)
                     else:
+                        # print("SetImg: 2c  -  RGB with without levels")
                         self.__image.setImage(
                             img,
                             # lut=None,
                             # levels=[[0,255], [0, 255], [0, 255]],
                             autoLevels=False)
+                    if maskimg is not None:
+                        # print(maskimg)
+                        if self.__overflowcolor:
+                            self.__maskImages[0].show()
+                            # print(self.__overflowcolor)
+                            smaskimg = np.sum(np.nan_to_num(maskimg), axis=2)
+                            smaskimg[smaskimg == 0] = np.nan
+                            self.__maskImages[0].setImage(
+                                np.array([
+                                    smaskimg.T
+                                    * self.__overflowcolor[0],
+                                    smaskimg.T
+                                    * self.__overflowcolor[1],
+                                    smaskimg.T
+                                    * self.__overflowcolor[2]
+                                ]).T,
+                                levels=[0, 255],
+                                autoLevels=False)
             elif (self.__autodisplaylevels
                   and self.__displaylevels[0] is not None
                   and self.__displaylevels[1] is not None):
                 self._hideimages()
                 self.__image.show()
+                # print("SetImg: 3")
                 self.__image.setImage(
                     img, autoLevels=False,
                     levels=self.__displaylevels,
@@ -596,16 +697,33 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
                   or self.__displaylevels[1] is None):
                 self._hideimages()
                 self.__image.show()
+                # print("SetImg: 4")
                 self.__image.setImage(
                     img, autoLevels=False,
                     autoDownsample=self.__autodownsample)
             else:
                 self._hideimages()
                 self.__image.show()
+                # print("SetImg: 5")
                 self.__image.setImage(
                     img, autoLevels=False,
                     levels=self.__displaylevels,
                     autoDownsample=self.__autodownsample)
+            if img is not None and len(img.shape) != 3:
+                if maskimg is not None:
+                    # print(maskimg)
+                    if self.__overflowcolor:
+                        self.__maskImages[0].show()
+                        # print(self.__overflowcolor)
+                        self.__maskImages[0].setImage(
+                            np.array([
+                                maskimg.T * self.__overflowcolor[0],
+                                maskimg.T * self.__overflowcolor[1],
+                                maskimg.T * self.__overflowcolor[2]
+                            ]).T,
+                            levels=[0, 255],
+                            autoLevels=False)
+
         except Exception as e:
             logger.warning(str(e))
             # print(str(e))
@@ -1300,6 +1418,9 @@ class ImageDisplayWidget(_pg.GraphicsLayoutWidget):
         """
         self.__images[1].hide()
         self.__images[2].hide()
+        self.__maskImages[0].hide()
+        self.__maskImages[1].hide()
+        self.__maskImages[2].hide()
 
     def setGradientColors(self, status=True):
         """ sets gradientcolors on/off
