@@ -27,11 +27,17 @@
 
 from .qtuic import uic
 import pyqtgraph as _pg
-from pyqtgraph import QtCore, QtGui
+from pyqtgraph import QtCore
 from .histogramWidget import HistogramHLUTWidget
 from . import messageBox
 from . import gradientDialog
 
+try:
+    from pyqtgraph import QtWidgets
+except Exception:
+    from pyqtgraph import QtGui as QtWidgets
+
+
 # from .histogramWidget import HistogramHLUTItem
 import math
 import os
@@ -55,7 +61,7 @@ _formclass, _baseclass = uic.loadUiType(
 logger = logging.getLogger("lavue")
 
 
-class LevelsGroupBox(QtGui.QWidget):
+class LevelsGroupBox(QtWidgets.QWidget):
 
     """
     Set minimum and maximum displayed values and its color.
@@ -86,7 +92,7 @@ class LevelsGroupBox(QtGui.QWidget):
         :param expertmode: expert mode flag
         :type expertmode: :obj:`bool`
         """
-        QtGui.QGroupBox.__init__(self, parent)
+        QtWidgets.QGroupBox.__init__(self, parent)
 
         #: (:class:`Ui_LevelsGroupBox') ui_groupbox object from qtdesigner
         self.__ui = _formclass()
@@ -576,7 +582,7 @@ class LevelsGroupBox(QtGui.QWidget):
         :type step: :obj:`str`
         """
         try:
-            fstep = float(step)
+            fstep = float(step or 0)
             if fstep <= 0:
                 fstep = None
                 self.__ui.stepLineEdit.setText("")
@@ -596,7 +602,7 @@ class LevelsGroupBox(QtGui.QWidget):
         :type factor: :obj:`str`
         """
         try:
-            ffactor = float(factor)
+            ffactor = float(factor or 0)
             if ffactor < 0:
                 ffactor = 0
                 self.__ui.autofactorLineEdit.setText("0")
@@ -624,7 +630,7 @@ class LevelsGroupBox(QtGui.QWidget):
             self.__hideControls()
             factor = str(self.__ui.autofactorLineEdit.text())
             try:
-                ffactor = float(factor)
+                ffactor = float(factor or 0)
                 if ffactor < 0:
                     ffactor = 0
                     self.__ui.autofactorLineEdit.setText("0")
@@ -739,11 +745,11 @@ class LevelsGroupBox(QtGui.QWidget):
         if minval >= 10e+20:
             minval = self.__minval
 
-        if maxval - minval <= 0:
-            if minval >= 1.:
-                minval = maxval - 1.
+        if maxval - minval < 0:
+            if minval >= 0.0:
+                minval = maxval
             else:
-                maxval = minval + 1
+                maxval = minval
 
         self.__ui.minDoubleSpinBox.setValue(minval)
         self.__ui.maxDoubleSpinBox.setValue(maxval)
@@ -879,7 +885,7 @@ class LevelsGroupBox(QtGui.QWidget):
             self.__channels = channels
         try:
             factor = str(self.__ui.autofactorLineEdit.text())
-            float(factor)
+            float(factor or 0)
             channels = None
             llim = None
             ulim = None
@@ -1246,12 +1252,12 @@ class LevelsGroupBox(QtGui.QWidget):
         name = self.__histograms[iid].gradient.name
 
         if name in self.__customgradients:
-            if QtGui.QMessageBox.question(
+            if QtWidgets.QMessageBox.question(
                     self, "Removing Label",
                     'Would you like  to remove "%s"" ?' %
                     (name),
-                    QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
-                    QtGui.QMessageBox.Yes) == QtGui.QMessageBox.No:
+                    QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
+                    QtWidgets.QMessageBox.Yes) == QtWidgets.QMessageBox.No:
                 return False
             self.__customgradients.pop(name)
             _pg.graphicsItems.GradientEditorItem.Gradients.pop(name)
