@@ -32,6 +32,8 @@ import sys
 import json
 from pyqtgraph import QtCore
 
+from .sardanaUtils import numpyEncoder
+
 if sys.version_info > (3,):
     unicode = str
 
@@ -57,6 +59,8 @@ class Settings(object):
         self.addrois = False
         #: (:obj:`bool`) fetch rois order enabled
         self.orderrois = False
+        #: (:obj:`bool`) fetch typed rois enabled
+        self.typedrois = False
         #: (:obj:`bool`) search for security stream port automatically
         self.secautoport = True
         #: (:obj:`bool`) show intensity hostogram
@@ -67,12 +71,16 @@ class Settings(object):
         self.showmask = False
         #: (:obj:`bool`) map image sources to color channels
         self.imagechannels = False
-        #: (:obj:`bool`) show high mask value  widget
+        #: (:obj:`bool`) show high mask value widget
         self.showhighvaluemask = False
+        #: (:obj:`bool`) show intensity overflow widget
+        self.showoverflow = False
         #: (:obj:`bool`) show statistics widget
         self.showstats = True
         #: (:obj:`bool`) show image step widget
         self.showsteps = True
+        #: (:obj:`bool`) show user plot widget
+        self.showuserplot = False
         #: (:obj:`bool`) calculate variance
         self.calcvariance = False
         #: (:obj:`bool`) show bakcground subtraction widget
@@ -123,6 +131,8 @@ class Settings(object):
         self.seccontext = zmq.Context()
         #: (:class:`zmq.Socket`) zmq security stream socket
         self.secsocket = self.seccontext.socket(zmq.PUB)
+        #: (:obj:`bool`) zmq colon configuration separator
+        self.zmqcolon = False
         #: (:obj:`bool`) security stream enabled
         self.secstream = False
         #: (:obj:`bool`) zero mask enabled
@@ -133,9 +143,11 @@ class Settings(object):
         self.negmask = False
         #: (:obj:`bool`) security stream options
         self.secsockopt = b""
-        #: (:obj:`float`) refresh rate is s
-        self.refreshrate = 0.2
-        #: (:obj:`float`) tool refresh rate time is s
+        #: (:obj:`float`) refresh time is s
+        self.refreshtime = 0.2
+        #: (:obj:`bool`) auto enlarge refresh time
+        self.autorefreshtime = True
+        #: (:obj:`float`) tool refresh time is s
         self.toolrefreshtime = 0.02
         #: (:obj:`float`) tool polling interval is s
         self.toolpollinginterval = 1.0
@@ -211,6 +223,8 @@ class Settings(object):
         self.asapobeamtime = ""
         #: (:obj:`str`) asapo sourcepath
         self.asaposourcepath = ""
+        #: (:obj:`str`) asapo beamline metadata file
+        self.asapobtmetafile = "/gpfs/current/beamtime-metadata-*.json"
         #: (:obj:`list` < :obj:`str` > ) asapo datasources
         self.asapodatasources = []
 
@@ -227,9 +241,15 @@ class Settings(object):
         #: (:obj:`str`) float type for pixel intensity
         self.floattype = "float"
 
+        #: (:obj:`str`) json list with overflow color
+        self.overflowcolor = "[255, 255, 255]"
+
         #: (:obj:`str`) json list with filters
         self.filters = "[]"
 
+        #: (:obj:`str`) json list with user functions
+        self.userfunctions = "[]"
+
         #: (:obj:`str`) json list with image source widget names
         self.imagesources = "[]"
 
@@ -278,6 +298,8 @@ class Settings(object):
         self.showallrois = False
         #: (:obj:`bool`) single rois flag
         self.singlerois = False
+        #: (:obj:`bool`) display roi aliases on the 2d image
+        self.labelrois = False
         #: (:obj:`bool`) send rois to LavueController flag
         self.sendrois = False
         #: (:obj:`bool`) send results to LavueController flag
@@ -384,6 +406,9 @@ class Settings(object):
         qstval = str(settings.value("Configuration/OrderROIs", type=str))
         if qstval.lower() == "true":
             self.orderrois = True
+        qstval = str(settings.value("Configuration/TypedROIs", type=str))
+        if qstval.lower() == "true":
+            self.typedrois = True
         qstval = str(settings.value("Configuration/ImageChannels", type=str))
         if qstval.lower() == "true":
             self.imagechannels = True
@@ -453,12 +478,19 @@ class Settings(object):
             "Configuration/ShowHighValueMaskWidget", type=str))
         if qstval.lower() == "true":
             self.showhighvaluemask = True
+        qstval = str(settings.value(
+            "Configuration/ShowOverflowWidget", type=str))
+        if qstval.lower() == "true":
+            self.showoverflow = True
         qstval = str(settings.value("Configuration/ShowStatistics", type=str))
         if qstval.lower() == "false":
             self.showstats = False
         qstval = str(settings.value("Configuration/ShowImageSteps", type=str))
         if qstval.lower() == "false":
             self.showsteps = False
+        qstval = str(settings.value("Configuration/ShowUserPlot", type=str))
+        if qstval.lower() == "true":
+            self.showuserplot = True
         qstval = str(settings.value("Configuration/CrossHairLocker", type=str))
         if qstval.lower() == "false":
             self.crosshairlocker = False
@@ -498,6 +530,10 @@ class Settings(object):
             "Configuration/SingleROIAliases", type=str))
         if qstval.lower() == "true":
             self.singlerois = True
+        qstval = str(settings.value(
+            "Configuration/LabelROIsWithAliases", type=str))
+        if qstval.lower() == "true":
+            self.labelrois = True
         qstval = str(settings.value("Configuration/SendROIs", type=str))
         if qstval.lower() == "true":
             self.sendrois = True
@@ -524,11 +560,12 @@ class Settings(object):
             except Exception:
                 for port in qstval:
                     int(port)
-                self.hidraport = json.dumps([str(tp) for tp in qstval])
+                self.hidraport = json.dumps(
+                    [str(tp) for tp in qstval], cls=numpyEncoder)
         else:
             try:
                 int(qstval)
-                self.hidraport = json.dumps([str(qstval)])
+                self.hidraport = json.dumps([str(qstval)], cls=numpyEncoder)
             except Exception:
                 pass
 
@@ -585,12 +622,27 @@ class Settings(object):
                 text = "lavue: Cannot connect to: %s" % self.secsockopt
                 status = [(text, value)]
 
+        qstval = str(settings.value(
+            "Configuration/ZMQColonSeparator", type=str))
+        if qstval.lower() == "true":
+            self.zmqcolon = True
+        else:
+            self.zmqcolon = False
+
         try:
-            self.refreshrate = float(
-                settings.value("Configuration/RefreshRate", type=str))
+            rt = settings.value("Configuration/RefreshTime", type=str)
+            if not rt:
+                rt = settings.value("Configuration/RefreshRate", type=str)
+            self.refreshtime = float(rt)
         except Exception:
             pass
 
+        qstval = str(
+            settings.value("Configuration/AutoEnlargeRefreshTime", type=str))
+        if qstval.lower() == "false":
+            self.autorefreshtime = False
+        elif qstval.lower() == "true":
+            self.autorefreshtime = True
         try:
             self.toolrefreshtime = float(
                 settings.value("Configuration/ToolRefreshTime", type=str))
@@ -654,7 +706,10 @@ class Settings(object):
             settings.value(
                 "Configuration/ZMQStreamTopics", type=str)
         if qstval:
-            self.zmqtopics = [str(tp) for tp in qstval]
+            try:
+                self.zmqtopics = json.loads(qstval)
+            except Exception:
+                self.zmqtopics = [str(tp) for tp in qstval]
 
         qstval2 = \
             settings.value(
@@ -663,9 +718,15 @@ class Settings(object):
             settings.value(
                 "Configuration/ASAPODataSources", type=str)
         if qstval:
-            self.asapodatasources = [str(tp) for tp in qstval]
+            try:
+                self.asapodatasources = json.loads(qstval)
+            except Exception:
+                self.asapodatasources = [str(tp) for tp in qstval]
         elif qstval2:
-            self.asapodatasources = [str(tp) for tp in qstval2]
+            try:
+                self.asapodatasources = json.loads(qstval2)
+            except Exception:
+                self.asapodatasources = [str(tp) for tp in qstval2]
         else:
             self.asapodatasources = []
 
@@ -701,6 +762,14 @@ class Settings(object):
         else:
             self.asaposourcepath = ""
 
+        qstval = \
+            settings.value(
+                "Configuration/ASAPOMetadataFile", type=str)
+        if qstval:
+            self.asapobtmetafile = str(qstval)
+        else:
+            self.asapobtmetafile = "/gpfs/current/beamtime-metadata-*.json"
+
         qstval = \
             settings.value(
                 "Configuration/HidraDetectorServers", type=str)
@@ -709,7 +778,8 @@ class Settings(object):
                 json.loads(qstval)
                 self.detservers = qstval
             except Exception:
-                self.detsetvers = json.dumps([str(tp) for tp in qstval])
+                self.detservers = json.dumps(
+                    [str(tp) for tp in qstval], cls=numpyEncoder)
 
         qstval = str(
             settings.value(
@@ -796,11 +866,21 @@ class Settings(object):
         if qstval:
             self.roiscolors = qstval
 
+        qstval = str(
+            settings.value("Configuration/IntensityOverflowColor", type=str))
+        if qstval:
+            self.overflowcolor = qstval
+
         qstval = str(
             settings.value("Configuration/Filters", type=str))
         if qstval:
             self.filters = qstval
 
+        qstval = str(
+            settings.value("Configuration/UserFunctions", type=str))
+        if qstval:
+            self.userfunctions = qstval
+
         qstval = str(
             settings.value(
                 "Configuration/SourceDisplayParams", type=str))
@@ -849,7 +929,7 @@ class Settings(object):
         except Exception:
             pass
         try:
-            self.diffnpt = float(
+            self.diffnpt = int(
                 settings.value("Tools/DiffractogramNPT", type=str))
         except Exception:
             pass
@@ -934,6 +1014,9 @@ class Settings(object):
         settings.setValue(
             "Configuration/OrderROIs",
             self.orderrois)
+        settings.setValue(
+            "Configuration/TypedROIs",
+            self.typedrois)
         settings.setValue(
             "Configuration/ImageChannels",
             self.imagechannels)
@@ -982,12 +1065,18 @@ class Settings(object):
         settings.setValue(
             "Configuration/ShowHighValueMaskWidget",
             self.showhighvaluemask)
+        settings.setValue(
+            "Configuration/ShowOverflowWidget",
+            self.showoverflow)
         settings.setValue(
             "Configuration/ShowStatistics",
             self.showstats)
         settings.setValue(
             "Configuration/ShowImageSteps",
             self.showsteps)
+        settings.setValue(
+            "Configuration/ShowUserPlot",
+            self.showuserplot)
         settings.setValue(
             "Configuration/CrossHairLocker",
             self.crosshairlocker)
@@ -995,8 +1084,11 @@ class Settings(object):
             "Configuration/CalculateVariance",
             self.calcvariance)
         settings.setValue(
-            "Configuration/RefreshRate",
-            self.refreshrate)
+            "Configuration/RefreshTime",
+            self.refreshtime)
+        settings.setValue(
+            "Configuration/AutoEnlargeRefreshTime",
+            self.autorefreshtime)
         settings.setValue(
             "Configuration/ToolRefreshTime",
             self.toolrefreshtime)
@@ -1021,6 +1113,9 @@ class Settings(object):
         settings.setValue(
             "Configuration/SecStream",
             self.secstream)
+        settings.setValue(
+            "Configuration/ZMQColonSeparator",
+            self.zmqcolon)
         settings.setValue(
             "Configuration/MaskingWithZeros",
             self.zeromask)
@@ -1086,10 +1181,10 @@ class Settings(object):
             self.toolwidgets)
         settings.setValue(
             "Configuration/ZMQStreamTopics",
-            self.zmqtopics)
+            json.dumps(self.zmqtopics or [], cls=numpyEncoder))
         settings.setValue(
             "Configuration/ASAPODataSources",
-            self.asapodatasources)
+            json.dumps(self.asapodatasources or [], cls=numpyEncoder))
         settings.setValue(
             "Configuration/ASAPOServer",
             self.asaposerver)
@@ -1102,6 +1197,9 @@ class Settings(object):
         settings.setValue(
             "Configuration/ASAPOSourcePath",
             self.asaposourcepath)
+        settings.setValue(
+            "Configuration/ASAPOMetadataFile",
+            self.asapobtmetafile)
         settings.setValue(
             "Configuration/HidraDetectorServers",
             self.detservers)
@@ -1165,9 +1263,15 @@ class Settings(object):
         settings.setValue(
             "Configuration/ROIsColors",
             self.roiscolors)
+        settings.setValue(
+            "Configuration/IntensityOverflowColor",
+            self.overflowcolor)
         settings.setValue(
             "Configuration/Filters",
             self.filters)
+        settings.setValue(
+            "Configuration/UserFunctions",
+            self.userfunctions)
         settings.setValue(
             "Configuration/SendROIs",
             self.sendrois)
@@ -1177,6 +1281,9 @@ class Settings(object):
         settings.setValue(
             "Configuration/SingleROIAliases",
             self.singlerois)
+        settings.setValue(
+            "Configuration/LabelROIsWithAliases",
+            self.labelrois)
         settings.setValue(
             "Configuration/ShowAllROIs",
             self.showallrois)
